/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GeoPlot tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GeoPlot tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant NCR-9711092, and by CAIDA members.

Report bugs and suggestions to ram@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    GeoPlot.java
**
**     Purpose:                 GeoPlot is a light-weight java applet which al-
**                              lows users to create a geographical image of a
**                              data set. The applet provides the user with 
**                              many options to represent the data set.
**                              Basically, GeoPlot plots a set of nodes and a
**                              set of lines that connect these nodes on an 
**                              image specified by the user.
**
**                              Read the README or http://www.caida.org/Tools
**                              /GeoPlot for more details on the different
**                              features and paramaters.
**
**                              This is the main file that draws the image and 
**                              uses all the other java files in the package.
**                   
**     Parameters:              mapname, bgcolor, fgcolor, title, date, 
**                              default_node_size, default_line_size,
**                              default_node_color, default_line_color, 
**                              default_font_size, refresh_interval, hide_key,
**                              top_lat, top_lon, bot_lat, bot_lon, color_key,
**                              size_key, data.
**
**                              Ihe parameters to this file are specified in  
**                              the applet tag:
**
**     Error Conditions:        -Image not found
**                              -Applet tag specified wrongly and class files 
**                               can't be loaded
**                              -Latitude/Longitude values are not specified 
**                               correctly
**
**     Status of Code:          GeoPlot's code does everything that's specified
**                              in the documentation at this point. Its not 
**                              been fully stress tested with "wrongly
**                              formatted" input. PENDING - ERROR HANDLING 
**                              THROUGH OUT CODE. If you happen to find any
**                              bugs or would like to suggest a new feature, 
**                              please send in your comments to ram@caida.org
** 
**     Author/Date:             Ram Periakaruppan, July 21st, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      The Source For Java Technology, http://www.java.sun.com.
**      Huffaker B., Otter, www.caida.org/Tools/Otter 
**      Periakaruppan R., GTrace, www.caida.org/Tools/GTrace
**
******************************************************************************/

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.applet.*;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.NoSuchElementException;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.io.InputStream;


public class GeoPlot extends Applet implements MouseMotionListener, 
					       MouseListener, Runnable
{
     Image map = null;                          /* image buffer that holds 
                                                   plain image
						*/
     Image mapbuffer = null;                    /* image buffer on which 
						   everything is drawn
						*/
     boolean error = false;                     /* flag that is set to indicate
						   an error
						*/ 
     boolean colorkey = false;                  /* flag indicating precense 
						   of color key
						*/
     boolean sizekey = false;                   /* flag indicating precense of
						   size key
						*/
     boolean mapLoaded = false;                 /* indicates status of the 
						   image load
						*/
     String title;                              /* title specified by the 
						   title applet parameter 
						*/
     String date;                               /* date specified by the date 
						   applet parameter
						*/
     static Color fgcolor;                      /* foreground color specified 
						   by the fgcolor applet 
						   parameter
						*/
     int default_node_size;                     /* default node size specified
						   by the default_node_size 
						   applet parameter
						*/
     int default_line_size;                     /* default line size specified
						   by the default_line_size 
						   applet parameter
						*/
     String default_node_color;                 /* default node color specified
						   by the default_node_color 
						   applet parameter
						*/
     String default_line_color;                 /* default line color specifie
						   by the default_line_color 
						   applet parameter
						*/
     int fontSize;                              /* fontSize specified by the 
						   fontSize applet parameter
						*/
     int refresh_interval;                      /* refresh interval specified 
						   by the refresh_interval 
						   applet parameter
						*/
     boolean hide_key;                          /* flag that decides whether 
						   size key and color key get 
						   displayed
						*/
     Thread t;                                  /* thread that runs the 
						   refresh interval timer
						*/
     float aLat;                                /* latitude factor used for 
						   conversion from lat/lon to 
						   point
						*/
     float aLon;                                /* longitude factor used for 
						   conversion from lat/lon to 
						   point
						*/
     float bLat;                                /* latitude constant used for 
						   conversion from lat/lon to 
						   point
						*/
     float bLon;                                /* longitude constant used for
						   conversion from lat/lon to 
						   point
						*/
     float top_lat=0;                           /* latitude at the top-left 
						   hand corner of the image
						*/
     float top_lon=0;                           /* longitude at the top-left 
						   hand corner of the image
						*/
     float bot_lat=0;                           /* latitude at the bottom-right
						   hand corner of the image
						*/
     float bot_lon=0;                           /* longitude at the bottom-
						   right hand corner of the 
						   image
						*/
     int Ckey_x=0;                              /* x-coordinate of the top-left
						   hand corner of the color key
						*/
     int Ckey_y=0;                              /* y-coordinate of the top-left
						   hand corner of the color key
						*/
     int Ckey_width=100;                        /* width of the color key */
     int Ckey_height=100;                       /* height of the color key */
     int Ckey_fontSize=13;                      /* font-size with which title 
						   should be displayed
						*/
     String Ckey_title="";                      /* title of the color key */

     int Skey_x=0;                              /* x-coordinate of the top-left
						   hand corner of the size key
						*/
     int Skey_y=0;                              /* y-coordinate of the top-left
						   hand corner of the size key
						*/
     int Skey_width=100;                        /* width of the size key */
     int Skey_height=100;                       /* height of the size key */
     int Skey_fontSize=13;                      /* font-size with which title 
						   should be displayed
						*/
     String Skey_title="";                      /* title of the size key */

     String data;                               /* holds the data specified by 
						   the data applet parameter
						*/
     Vector lines = new Vector();               /* stores array of line objects
						*/
     Vector color_key = new Vector();           /* stores array of key objects
						*/
     Vector size_key = new Vector();            /* stores array of key objects
						*/
     Vector matched_regions = new Vector();     /* number of regions in which 
						   the mouse pointer could be 
						   over a line/node
						*/
     Hashtable nodes = new Hashtable();         /* stores a hash of the nodes 
						   specified by the data 
						   parameter
						*/
     Hashtable lineMatrix = new Hashtable();    /* indicates the num. of lines
						   in the quadrants surrounding
						   each node
						*/
     Node node;                                 /* a node object */
     Line line;                                 /* a line object */
     Key key;                                   /* a key object */

     static Rectangle r;                        /* holds the rectagular 
						   co-ordinates of the applet
						*/
     Graphics gc;                               /* graphics context used to 
						   paint to the image buffer
						*/
     FontMetrics fm;                            /* fontmetrics object used to 
						   find out attributes about 
						   current font
						*/

     StatusBar sb;                              /* statusbar object */
     boolean mouseOver = false;                 /* flag that is set when mouse
						   if over a node/line
						*/
     boolean bufferChanged = false;             /* flag that is set when mouse
						   is over a node/line to 
                                                   indicate contents of 
						   mapBuffer has changed
						*/
     NodLin nodLin = null;                      /* nodlin object */
     Point mouseP = new Point();                /* current mouse pointer 
						   location
						*/

     MediaTracker tracker;                      /* tracker object to track 
						   loading of map
						*/
     boolean paintDataCalled=false;             /* flag indicates whether 
						   method paintData was 
						   invoked
						*/

     /* the following constants define the various quadrant combinations */ 

     final static int quadone = 0;
     final static int quadtwo = 1;
     final static int quadthree = 2;
     final static int quadfour = 3;
     final static int quadfive = 4;
     final static int quadsix = 5;
     final static int quadseven = 6;
     final static int quadeight = 7;
     final static int quadfivesix = 56;
     final static int quadtwoone = 21;
     final static int quadsevenzero = 70;
     final static int quadfourthree = 43;


     final static int KEY_DATA_SIZE = 3;

/******************************************************************************
**
**     Method name:             getAppletInfo (class GeoPlot)
**
**     Purpose:                 returns information about the purpose of the 
**                              applet
**    
**     Parameters:              NONE
**
**     Return Values:           String - the applet info
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String getAppletInfo()
     {
	  return "A general purpose Geographical Visualization Tool";
     }

/******************************************************************************
**
**     Method name:             init (class GeoPlot)
**
**     Purpose:                 Applet Initialization - loads image,
**                              parses all applet parameters and stores them,
**                              calculates applet lat/lon constants.
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Side effects:            Sets error flag if image can't be loaded or if
**                              latitudes/longitudes are incorrect. Error flag
**                              triggers paint method to give the user a error
**                              message.
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void init()
     {
	  String param;                     /* stores data obtained from 
					       applet parameters
					    */
	  String str;                       /* holds tokens returned by 
					       StringTokenizer
					    */
	  Color bgcolor;                    /* back ground color specified by 
					       the bgcolor applet parameter
					    */
	  StringTokenizer tokenizer;        /* tokenizer used for parsing 
					       parameters
					    */
	  StringTokenizer fields;           /* tokenizer used for parsing 
					       parameters
					    */

	  /* instantiate the status bar object */
	  sb = new StatusBar(this);

	  /* Create mapbuffer image to draw on */
	  r = getBounds();
	  mapbuffer = createImage(r.width, r.height);
	  gc = mapbuffer.getGraphics();
	  tracker = new MediaTracker(this);

	  /* Setup mouse listeners */
	  addMouseMotionListener(this);
	  addMouseListener(this);

	  /* Get the map name and proceed if it exists */
	  param = getParameter("mapname");
	  if (param != null)
	  {
	       /* Check if the mapname is a URL */
	       if (param.toLowerCase().startsWith("http://"))
	       {
		    try
		    {
			 map = getImage(new URL(param));
		    }
		    catch(Exception e)
		    {
			 error = true;
			 repaint();
		    }
	       }
	       else
	       {
		    map = getImage(getDocumentBase(), param);
	       }
	       
	       tracker.addImage(map, 1);
	       
	       /* Parse background color parameter */
	       param = getParameter("bgcolor");
	       if (param != null)
	       {
		    bgcolor = NodLin.getColor(param);
		    if (bgcolor == null)
		    {
			 bgcolor = Color.white;
		    }
	       }
	       else
	       {
		    bgcolor = Color.white;
	       }

	       setBackground(bgcolor);

	       /* Parse foreground color parameter */
	       param = getParameter("fgcolor");
	       if (param != null)
	       {
		    fgcolor = NodLin.getColor(param);
		    if (bgcolor == null)
		    {
			 fgcolor = Color.black;
		    }
	       }
	       else
	       {
		    fgcolor = Color.black;
	       }

	       /* Parse title parameter */
	       param = getParameter("title");
	       if (param != null)
	       {
		    title = param;
	       }
	       else
	       {
		    title = "";
	       }

	       /* Parse date parameter */
	       param = getParameter("date");
	       if (param != null)
	       {
		    date = param;
	       }
	       else
	       {
		    date = "";
	       }

	       /* Parse refresh interval parameter */
	       param = getParameter("refresh_interval");
	       try
	       {
		    if (param != null)
		    {
		         refresh_interval = Integer.valueOf(param).intValue();
		    }
		    else
		    {
			 refresh_interval = 0;
		    }
	       }
	       catch(NumberFormatException e)
	       {
		    refresh_interval = 0;
	       }

	       /* Parse default node size parameter */
	       param = getParameter("default_node_size");
	       try
	       {
		    if (param != null)
		    {
			 default_node_size = Integer.valueOf(param).intValue();
		    }
		    else
		    {
			 default_node_size = 6;
		    }
	       }
	       catch(NumberFormatException e)
	       {
		    System.err.println
			("Handle numberformatexception default_node_size");
	       }

	       /* Parse default line size parameter */
	       param = getParameter("default_line_size");
	       try
	       {
		    if (param != null)
		    {
		         default_line_size = Integer.valueOf(param).intValue();
		    }
		    else
		    {
			 default_line_size = 1;
		    }
	       }
	       catch(NumberFormatException e)
	       {
		    System.err.println
			("Handle numberformatexception default_line_size");
	       }

	       /* Parse default node color parameter */
	       param = getParameter("default_node_color");
	       if (param != null)
	       {
		    default_node_color = param;
	       }
	       else
	       {
		    default_node_color = "black";
	       }

	       /* Parse default line color parameter */
	       param = getParameter("default_line_color");
	       if (param != null)
	       {
		    default_line_color = param;
	       }
	       else
	       {
		    default_line_color = "red";
	       }

	       /* Parse default font size parameter */
	       param = getParameter("default_font_size");
	       try
	       {
		    if (param != null)
		    {
			 fontSize = Integer.valueOf(param).intValue();
		    }
		    else
		    {
			 fontSize = 10;
		    }
	       }
	       catch(NumberFormatException e)
	       {
		    fontSize = 10;
	       }	       

	       /* Parse hide key parameter */
	       param = getParameter("hide_key");
	       hide_key = new Boolean(param).booleanValue();

	       /* Parse top latitude parameter */
	       param = getParameter("top_lat");
	       try
	       {
		    if (param != null)
		    {
			 top_lat = Float.valueOf(param).floatValue();
		    }
		    else
		    {
			 error = true;
			 repaint();
		    }
	       }
	       catch(NumberFormatException e)
	       {
		    System.err.println("Handle numberformatexception top_lat");
	       }

	       /* Parse top longitude parameter */
	       param = getParameter("top_lon");
	       try
	       {
		    if (param != null)
		    {
			 top_lon = Float.valueOf(param).floatValue();
		    }
		    else
		    {
			 error = true;
			 repaint();
		    }
	       }
	       catch(NumberFormatException e)
	       {
		    System.err.println("Handle numberformatexception top_lon");
	       }

	       /* Parse bottom latitude parameter */
	       param = getParameter("bot_lat");
	       try
	       {
		    if (param != null)
		    {
			 bot_lat = Float.valueOf(param).floatValue();
		    }
		    else
		    {
			 error = true;
			 repaint();
		    }
	       }
	       catch(NumberFormatException e)
	       {
		    System.err.println("Handle numberformatexception bot_lat");
	       }

	       /* Parse bottom longitude parameter */
	       param = getParameter("bot_lon");
	       try
	       {
		    if (param != null)
		    {
			 bot_lon = Float.valueOf(param).floatValue();
		    }
		    else
		    {
			 error = true;
			 repaint();
		    }
	       }
	       catch(NumberFormatException e)
	       {
		    System.err.println("Handle numberformatexception bot_lon");
	       }
		  

	       /* Parse color key parameter creating a key object for each 
		  interval and adding the objects to the color key vector.
		  Check the documentation to see what each parameter is used 
		  for.
	       */

	       param = getParameter("color_key");
	       if (param != null)
	       {
		    colorkey = true;
		    tokenizer = new StringTokenizer(param,";");
		    while(tokenizer.hasMoreTokens())
		    {
			 str =  tokenizer.nextToken();

			 if (str.startsWith("key"))
			 {
			      char ch;
			      StringTokenizer key;
			      fields = new StringTokenizer(str,",");
			      fields.nextToken();
				 
			      while(fields.hasMoreTokens())
			      {
				   str =  fields.nextToken();
				   str =  str.trim();
				   ch = Character.toLowerCase(str.charAt(0));

   try
   {
       key = new StringTokenizer(str,":");
       key.nextToken();				       
       
       switch(ch)
       {
           case 'x':
	       try
	       {
		   Ckey_x = Integer.valueOf(key.nextToken()).intValue();
	       }
	       catch(NumberFormatException nfe) { }
	       break;
		    
           case 'y':
	       try
	       {
		   Ckey_y = Integer.valueOf(key.nextToken()).intValue();
	       }
	       catch(NumberFormatException nfe) { }
	       break;
		    
           case 'w':
	       try
	       {
		   Ckey_width = Integer.valueOf(key.nextToken()).intValue();
	       }
	       catch(NumberFormatException nfe) { }
	       break;
							    
           case 'h':
	       try
	       {
		   Ckey_height = Integer.valueOf(key.nextToken()).intValue();
	       }
	       catch(NumberFormatException nfe) { }
	       break;
	       
           case 't':
	       try
	       {
		   Ckey_title = key.nextToken();
	       }
	       catch(NoSuchElementException nsee) { }
	       break;
	       
           case 's':
	       try
	       {
		   Ckey_fontSize = Integer.valueOf(key.nextToken()).intValue();
	       }
	       catch(NumberFormatException nfe) { }
	       break;
       }
   }
   catch(NoSuchElementException nsee) { }
			      }
			 }
			 else
			 {
			      color_key.addElement(new Key(str.trim()));
			 }
		    }
	       }
		   
	       /* Parse size key parameter creating a key object for each 
		  interval and adding the objects to the size key vector.
		  Check the documentation to see what each parameter is used 
		  for.
	       */

	       param = getParameter("size_key");
	       if (param != null)
	       {
		    sizekey = true;
		    tokenizer = new StringTokenizer(param,";");
		    while(tokenizer.hasMoreTokens())
		    {
			 str =  tokenizer.nextToken();

			 if (str.startsWith("key"))
			 {
			      char ch;
			      StringTokenizer key;
			      fields = new StringTokenizer(str,",");
			      fields.nextToken();
				 
			      while(fields.hasMoreTokens())
			      {
				   str =  fields.nextToken();
				   str =  str.trim();
				   ch = Character.toLowerCase(str.charAt(0));

      try
      {
	  key = new StringTokenizer(str,":");
	  key.nextToken();				       
	  
	  switch(ch)
	  {
	      case 'x':
		  try
		  {
		      Skey_x = Integer.valueOf(key.nextToken()).intValue();
		  }
		  catch(NumberFormatException nfe) { }
		  break;
		  
	      case 'y':
		  try
		  {
		      Skey_y = Integer.valueOf(key.nextToken()).intValue();
		  }
		  catch(NumberFormatException nfe) { }
		  break;
		  
	      case 'w':
		  try
		  {
		      Skey_width = Integer.valueOf(key.nextToken()).intValue();
		  }
		  catch(NumberFormatException nfe) { }
		  break;
		  
	      case 'h':
		  try
		  {
		      Skey_height = Integer.valueOf(
						  key.nextToken()).intValue();
		  }
		  catch(NumberFormatException nfe) { }
		  break;
							    
	      case 't':
		  try
		  {
		      Skey_title = key.nextToken();
		  }
		  catch(NoSuchElementException nsee) { }
		  break;
		  
	      case 's':
		  try
		  {
		      Skey_fontSize = Integer.valueOf(
						  key.nextToken()).intValue();
		  }
		  catch(NumberFormatException nfe) { }
		  break;
	  }
      }
      catch(NoSuchElementException nsee) { }
			      }
			 }
			 else
			 {
			      size_key.addElement(new Key(str.trim()));
			 }
		    }
	       }

	       /* Check for the existance of data parameter and call function 
		  parseData to parse the data 
	       */
	       param = getParameter("data");
	       if (param != null)
	       {
		    data = param;
		    parseData(data);
	       }
	       else
	       {
		   /* Handle condition when user has not given a data 
		      parameter in applet
		   */
	       }

	       /* initialize lineMatrix for each node */
	       buildLineMatrix();

	       /* calculate the constants used for converting from lat/lon 
		  to a point on the image 
	       */
	       bLon = top_lon;
	       bLat = top_lat;
	       aLon = (bot_lon-top_lon)/r.width;
	       aLat = (bot_lat-top_lat)/r.height;
	  }
	  else
	  {
	       /* image load failed */
	       error = true; 
	       repaint();
	  }
     }
        
    
/******************************************************************************
**
**     Method name:             parseData (class GeoPlot)
**
**     Purpose:                 Parses the data creating a node and line object
**                              for node/line definition. If the data is 
**                              specified as a URL, then the func.reads the 
**                              file and then does the parsing.
**
**                              Function ignores any badly formatted data.
**    
**     Parameters:              String locData - the data specified in the 
**                              applet data parameter.
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private void parseData(String locData)
     {
	  String str;                       /* holds tokens returned by 
					       StringTokenizer
					    */
	  String field[] = new String[4];   /* holds the indiviual fields of 
					       the node/line definitions
					    */
	  StringTokenizer tokenizer;        /* tokenizer used for parsing 
					       parameters
					    */
	  StringTokenizer fields;           /* tokenizer used for parsing 
					       parameters
					    */

	  /* Check if data is given as a URL and if it is, make a 
	     connection and read contents of data file
	  */
	  if (locData.toLowerCase().startsWith("http://"))
	  {
	       int c;                 /* holds each character read from the 
					 input stream.
				      */ 
	       URL data;              /* URL to data file */
	       URLConnection dataCon; /* connection to that data file */
	       StringBuffer sb;       /* buffer to which each character is 
					 added
				      */
			 
	       try
	       {
		    data = new URL(locData);
		    dataCon = data.openConnection();

		    int len = dataCon.getContentLength();
		    
		    if (len > 0)
		    {
			 sb = new StringBuffer(len);
			 InputStream input = dataCon.getInputStream();
			 int i = len;
			 while ((( c = input.read()) != -1) && (--i > 0))
			 {
			      sb.append((char) c);
			 }
			 locData = sb.toString();
			 input.close();
		    }
	       }
	       catch(Exception e)
	       {
		    locData = null;
	       }
	  }

	  /* Parse contents of locData adding the node and line objects to the
	     respective arrays 
	  */
	  if (locData != null)
	  {
	       int numofTokens = 0;  /* number of tokens in each node/line 
					definition
				     */
	       int numofNodes = 0;   /* keeps track of the number of nodes */
	       char ch;              /* holds the first character of each data 
					line
				     */

	       /* each node/line defintion should be terminated by a 
		  semi-colon 
	       */
	       tokenizer = new StringTokenizer(locData,";");
	       while(tokenizer.hasMoreTokens())
	       {
		    str =  tokenizer.nextToken();
		    str = str.trim();
		    ch = Character.toLowerCase(str.charAt(0));
		    if ( ch == 'n')
		    {
			 fields = new StringTokenizer(str);
			 numofTokens = fields.countTokens();
			 fields.nextToken();
			 if (numofTokens < 4)
			 {
			      // Bad data handle case
			 }
			 else
			 {
			      for (int i=0; i<3; i++)
			      {
				   field[i] = fields.nextToken();
			      }
					
			      field[3] = "";
					
			      while(fields.hasMoreTokens())
			      {
				   field[3] = field[3] + " " + 
				                            fields.nextToken();
			      }
				   
			      if (field[3].length() > 0)
			      {
				   node = new Node(numofNodes++, field[1], 
						   field[2], default_node_size,
						   default_node_color, field[3]
						   );
			      } 
			      else
			      {
				   node = new Node(numofNodes++, field[1], 
						   field[2], default_node_size,
						   default_node_color);
			      } 
					
			      nodes.put(field[0],node);
			 }
		    }
		    else if (ch == 'l')
		    {
			 fields = new StringTokenizer(str);
			 numofTokens = fields.countTokens();
			 fields.nextToken();
			 if (numofTokens < 3)
			 {
			      // Bad data handle case
			 }
			 else
			 {
			      for (int i=0; i<2; i++)
			      {
				   field[i] = fields.nextToken();
			      }
			      field[2] = "";
			      while(fields.hasMoreTokens())
			      {
				   field[2] = field[2] + " " + 
				                            fields.nextToken();
			      }
			      
			      if (field[2].length() > 0)
			      { 
				   line = new Line(field[0], field[1], 
						   default_line_size, 
						   default_line_color, 
						   field[2]);
			      }
			      else
			      {
				   line = new Line(field[0], field[1], 
						   default_line_size, 
						   default_line_color);
			      }
				   
			      lines.addElement(line);
			 }
		    }
		    else
		    {
			/* Handle condition where data is niether a node 
			   nor line
			*/
		    }
	       }
	  }	 
     }

/******************************************************************************
**
**     Method name:             run (class GeoPlot)
**
**     Purpose:                 run() is called when the thread t is started.
**                              Thread sleeps for the specified duration of
**                              seconds. If thread is interrupted it restarts,
**                              if it wakes up, then it clears the present 
**                              image buffer, clears all data structures, 
**                              reloads the data, and reconstructs the image.
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void run()
     {
	  try
	  {
	       t.sleep(1000 * refresh_interval);
	  }
	  catch(InterruptedException ie) 
	  {
	       t = new Thread(this);
	       t.start();
	  }

	  lines.removeAllElements();
	  nodes.clear();
	  lineMatrix.clear();
	  mapbuffer = createImage(r.width, r.height);
	  gc = mapbuffer.getGraphics();
	  parseData(data);
	  buildLineMatrix();
	  paintData(gc);
	  t = new Thread(this);
	  t.start();
     }

/******************************************************************************
**
**     Method name:             start (class GeoPlot)
**
**     Purpose:                 The start() method is called after init(). Also
**                              called whenever the applet is restarted. It
**                              checks if the refresh interval parameter is set
**                              and creates a new thread t.
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void start()
     {
	  if (refresh_interval > 0)
	  {
	       t = new Thread(this);
	       t.start();
	  }
     }

/******************************************************************************
**
**     Method name:             stop (class GeoPlot)
**
**     Purpose:                 This method is called when the web browser 
**                              leaves the HTML document containing the applet.
**                              The method stops the execution of the seperate
**                              thread t.
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/
    
     public void stop()
     {
	  if ( refresh_interval > 0)
	  {
	       t.stop(); 
	       t=null;
	  }
     }

/******************************************************************************
**
**     Method name:             update (class GeoPlot)
**
**     Purpose:                 This method is called when the applet requests
**                              that a portion of the window be redrawn. It 
**                              inturn invokes the paint method so that user
**                              dosen't experience a flash of the default 
**                              background each time..
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           graphics environment applet is 
**                                           running in.
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void update(Graphics g)
     {
	  paint(g);
     }

/******************************************************************************
**
**     Method name:             paint (class GeoPlot)
**
**     Purpose:                 This method is called when the applet's window
**                              must be restored. Also whenever a repaint() is
**                              called. Method indicates to user if error flag 
**                              is set.
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the  
**                                           applet is running in.
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void paint(Graphics g)
     {
	   // xxx - pending error flag is also set for bad lat/lons.

	  if (error)
	  {
	       String errorMsg = "Image Could Not Be Loaded";
	       g.setColor(Color.red);
	       g.fillRect(0,0,r.width,r.height);
	       g.setColor(Color.black);
	       Font font = new Font("Dialog",Font.BOLD,20); 
	       g.setFont(font);
	       fm = g.getFontMetrics();
	       g.drawString(errorMsg, r.width/2 - fm.stringWidth(errorMsg)/2, 
			    r.height/2);
	  }
	  else
	  {      
	       /* if image is not fully loaded paint whatever has 
		  been loaded 
	       */
	       if (!mapLoaded)
	       {
		    g.drawImage(map, 0, 0, r.width, r.height, this);

		    if (tracker.checkID(1, true))
		    {
			if (!paintDataCalled)
			{
			    paintData(gc);
			}
			mapLoaded = true;
		    }
	       }
	       else if (mouseOver)  /* if the mouseOver flag is set, 
				       redraw that node/line 
				    */
	       {
		    drawNodLin(g);
	       }
	       else  /* when image is fully loaded draw map buffer 
			that contains the node/lines also 
		     */
	       {
		    g.drawImage(mapbuffer, 0,0, r.width, r.height,this);
	       }
	  }
     }
        
/******************************************************************************
**
**     Method name:             imageUpdate (class GeoPlot)
**
**     Purpose:                 Override default implementation of imageUpdate
**                              to allow for smooth image loading.
**    
**     Parameters:              Image img - the image being loaded
**                              int flags - communicates the status of the 
**                                          update report 
**                              int x,y,w,h - represents a rectangle that 
**                                            contains different
**                                            values depending on the values 
**                                            passed in flags.
**
**     Return Values:           boolean
**                                           
**     Side effects:            NONE
**
**     Last Modified:           June 18th, 1999
**
**     Code From :              Naughton, P., Schildt, H., JAVA - The Complete
**                              Reference. Pg. 658
**
******************************************************************************/

     public boolean imageUpdate(Image img, int flags, int x, int y, 
				                                  int w, int h)
     {    
	  if (( flags & ALLBITS) != 0)             // entire image loaded
	  {
	       repaint();
	       if (!paintDataCalled)
	       {
		   paintData(gc);
	       }
	       mapLoaded = true;
	  }
	  else if ((flags & SOMEBITS) != 0)        // new partial data
	  {
	       repaint(x,y,w,h);                   // paint new pixels
	  }
	  else if (( flags & (ABORT|ERROR)) != 0)  
	  {
	       error = true;                       // file not found
	       repaint();
	  }

	  return (flags & (ALLBITS|ABORT|ERROR)) == 0;
     }
    
/******************************************************************************
**
**     Method name:             getNodeLineSize (class GeoPlot)
**
**     Purpose:                 Match the size value of a node/line with the
**                              size key and return the size indicated by
**                              the size key.
**    
**     Parameters:              NodLin ln      - superclass object of node/line
**                                               that is referencing a node/
**                                               line object. 
**                              int numofSkeys - number of size key ranges
**
**     Return Values:           int - the diameter of nodes, width for lines.
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private int getNodeLineSize(NodLin ln, int numofSkeys)
     {
	  int nodlinSize = ln.getSize();
	  if (sizekey && ln.getSValue() != Float.POSITIVE_INFINITY)
	  {
	       for(int sk=0; sk < numofSkeys; sk++)
	       {
		    key = (Key)size_key.elementAt(sk);
		    if (key.withinbounds(ln.getSValue()))
		    {
			 try
			 {
			      nodlinSize = Integer.valueOf(
						    key.getValue()).intValue();
			 }
			 catch(NumberFormatException nfe) { }
			 break;
		    }
	       }
	  }

	  return nodlinSize;
     }

/******************************************************************************
**
**     Method name:             setColor (class GeoPlot)
**
**     Purpose:                 Match the color value of the node/line against
**                              the color key and set the color to that color
**                              key value.
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the 
**                                           applet is running in.
**                                            
**                              NodLin ln  - superclass object of node/line
**                                           that is referencing a node/line 
**                                           object.
**                              int numofCkeys - number of color key ranges
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private void setColor(Graphics g, NodLin ln, int numofCkeys)
     {
	  g.setColor(ln.getColor());

	  if (colorkey && ln.getCValue() != Float.POSITIVE_INFINITY)
	  {
	       for(int ck=0; ck < numofCkeys; ck++)
	       {
		    key = (Key)color_key.elementAt(ck);
		    if (key.withinbounds(ln.getCValue()))
		    {
			 setColor(g,key.getValue());
			 break;
		    }
	       }
	  }
     }

/******************************************************************************
**
**     Method name:             setColor (class GeoPlot)
**
**     Purpose:                 set the color to the value specified in the 
**                              parameter.
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the
**                                           applet is running in. 
**                              String color - the color expressed either as a 
**                                             RGB value or word.
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private void setColor(Graphics g, String color)
     {
	  StringTokenizer tokenizer;

	  if (!color.substring(0,3).toLowerCase().startsWith("rgb"))
	  {
	       g.setColor((Color)NodLin.colormap.get(color));
	  }
	  else
	  {
	       tokenizer = new StringTokenizer(color,"-");
	       if (tokenizer.countTokens() < 4)
	       {
		    g.setColor(fgcolor);
		    return;
	       }
	       
	       tokenizer.nextToken();
	       try
	       {
		    g.setColor(new Color(Integer.valueOf(
					     tokenizer.nextToken()).intValue(),
					 Integer.valueOf(
					     tokenizer.nextToken()).intValue(),
					 Integer.valueOf(
					     tokenizer.nextToken()).intValue())
			       );
	       }
	       catch(NumberFormatException e)
	       {
		    System.err.println
		      ("Handle numberformatexception color_key setting color");
		    g.setColor(fgcolor);
		    return;
	       }
	  }
     }

/******************************************************************************
**
**     Method name:             getRectPoints (class GeoPlot)
**
**     Purpose:                 Given a line, the method determines the 
**                              boundary points that enclose the line by a
**                              rectangular region, the distance from the line
**                              to the top edge and the dist. from the line
**                              to the bottom edge are also specified as 
**                              parameters.
**    
**     Parameters:              Point p1 - start point of the line
**                              Point p2 - end point of the line
**                              float minusH - distance on the lower side of 
**                                             line
**                              float plusH - distance on the upper side of 
**                                            line 
**
**     Return Values:           Point[] - the boundary points of the 
**                                        rectangular region.
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private static Point[] getRectPoints(Point p1, Point p2, 
					  float minusH, float plusH)
     {
	  Point p[] = new Point[2];
	  p[0] = new Point(0,0);
	  p[1] = new Point(0,0);
	  
	  float angle = new Double(Math.PI / 2.0).floatValue();

	  double theta = Math.atan(new Float(
				       Math.abs(p1.y - p2.y)).floatValue()/
				   new Float(
				       Math.abs(p1.x - p2.x)).floatValue());

          int quad = getQuadP1(p1,p2);

	  if ( quad == quadtwo || quad == quadfour)
	  {
	       theta = Math.PI / 2 - theta;
	  }

	  int x1 = new Double(Math.round( 
			       minusH * Math.cos(theta + angle))).intValue(); 
	  int y1 = new Double(Math.round( 
			       minusH * Math.sin(theta + angle))).intValue();
	  int x2 = new Double(Math.round( 
			      plusH * Math.cos(angle - theta))).intValue(); 
	  int y2 = new Double(Math.round( 
			      plusH * Math.sin(angle - theta))).intValue();

	  switch(quad)
	  {
	       case quadone:
		               p[0].x = p1.x+x1;
			       p[0].y = p1.y-y1;
			       p[1].x = p1.x+x2;
			       p[1].y = p1.y+y2;		               
		               break;
 	       case quadtwo: 
		               p[0].x = p1.x-y1;
			       p[0].y = p1.y-x1;
			       p[1].x = p1.x+y2;
			       p[1].y = p1.y-x2;
		               break;
	       case quadthree:
		               p[1].x = p1.x-x1;
			       p[1].y = p1.y+y1;
			       p[0].x = p1.x-x2;
			       p[0].y = p1.y-y2;
		               break;
	       case quadfour:
		               p[1].x = p1.x+y1;
			       p[1].y = p1.y+x1;
			       p[0].x = p1.x-y2;
			       p[0].y = p1.y+x2;
	  }

	  return p;
     }

/******************************************************************************
**
**     Method name:             getArrowPoints (class GeoPlot)
**
**     Purpose:                 Given a line, this method determines the 
**                              endpoints of an arrow that would be formed at 
**                              the p1 end of the line.
**    
**     Parameters:              Point p1 - start point of the line
**                              Point p2 - end point of the line 
**                              float length - length of the arrow 
**                              float angle - angle between the two lines that 
**                                            form the shape of the arrow. 
**
**     Return Values:           Point[] - the endpoints of both lines that
**                                        makeup the arrow.
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public static Point[] getArrowPoints(Point p1, Point p2, 
					             float length, float angle)
     {
	  Point p[] = new Point[2];
	  p[0] = new Point(0,0);
	  p[1] = new Point(0,0);
	  
	  angle = new Double(angle * Math.PI / 180.0).floatValue();

	  double h = length / Math.sin(angle);
	  double theta = Math.atan(new Float(
					Math.abs(p1.y - p2.y)).floatValue()/
				   new Float(
				        Math.abs(p1.x - p2.x)).floatValue());

          int quad = getQuadP1(p1,p2);

	  if ( quad == quadtwo || quad == quadfour)
	  {
	       theta = Math.PI / 2 - theta;
	  }

	  int x1 = new Double(Math.round( 
				  h * Math.cos(theta + angle))).intValue(); 
	  int y1 = new Double(Math.round( 
				  h * Math.sin(theta + angle))).intValue();
	  int x2 = new Double(Math.round( 
				  h * Math.cos(angle - theta))).intValue(); 
	  int y2 = new Double(Math.round( 
				  h * Math.sin(angle - theta))).intValue();

	  switch(quad)
	  {
	       case quadone:
		               p[0].x = p1.x+x1;
			       p[0].y = p1.y-y1;
			       p[1].x = p1.x+x2;
			       p[1].y = p1.y+y2;		               
		               break;
 	       case quadtwo: 
		               p[0].x = p1.x-y1;
			       p[0].y = p1.y-x1;
			       p[1].x = p1.x+y2;
			       p[1].y = p1.y-x2;
		               break;
	       case quadthree:
		               p[1].x = p1.x-x1;
			       p[1].y = p1.y+y1;
			       p[0].x = p1.x-x2;
			       p[0].y = p1.y-y2;
		               break;
	       case quadfour:
		               p[1].x = p1.x+y1;
			       p[1].y = p1.y+x1;
			       p[0].x = p1.x-y2;
			       p[0].y = p1.y+x2;
	  }

	  return p;
     }

/******************************************************************************
**
**     Method name:             drawArrow (class GeoPlot)
**
**     Purpose:                 This method draws a line from point p1 to p2,
**                              if the thickness of the line > 1, then it draws
**                              a filled in triangle.
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the 
**                                           applet is running in. 
**                              int lineSize - width of the line
**                              Point p1 - start point of the line 
**                              Point p2 - end point of the line 
**                              int topBtm - flag that inidcates direction of 
**                                           arrow
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public static void drawArrow(Graphics g,int lineSize,Point p1, 
				                         Point p2, int topBtm)
     {
	  int wd;
	  Point P1[] = new Point[2];

	  if( lineSize == 1)
	  {
	       g.drawLine(p1.x,p1.y,p2.x,p2.y);
	       return;
	  }
	  else if (lineSize <= 6)
	  {
	       wd = lineSize * 2;
	  }
	  else
	  {
	       wd = lineSize;
	  }

	  int xPoints[] = new int[3];
	  int yPoints[] = new int[3];
	  
	  //P1 = getArrowPoints(p1, p2, lineSize, 45);
	  P1 = getArrowPoints(p1, p2, wd, 45);
	  xPoints[0] = p1.x; xPoints[1]=P1[topBtm].x; 
	  yPoints[0] = p1.y; yPoints[1]=P1[topBtm].y; 

	  //P1 = getArrowPoints(p1, p2, lineSize, 0);
	  P1 = getArrowPoints(p1, p2, wd, 0);
	  xPoints[2]=P1[topBtm].x;;
	  yPoints[2]=P1[topBtm].y;;
	  
	  g.fillPolygon(xPoints,yPoints,3);
     }

/******************************************************************************
**
**     Method name:             drawThickLine (class GeoPlot)
**
**     Purpose:                 This method is called from drawSLine, whenever
**                              a line of width > 1 is required to be drawn.
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the 
**                                           applet is running in.
**                              int lineSize - width of the line
**                              Point p1 - start point of the line 
**                              Point p2 - end point of the line 
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public static void drawThickLine(Graphics g, int lineSize, 
				                         Point p1, Point p2)
     {
	  Point P1[] = new Point[2];
	  Point P2[] = new Point[2];

	  int wd = lineSize / 2;
	       
	  if ((lineSize % 2) == 0)
	  {
	       P1 = getRectPoints(p1,p2,wd,wd);	
	       P2 = getRectPoints(p2,p1,wd,wd);			     
	  }
	  else
	  {
	       P1 = getRectPoints(p1,p2,wd,wd+1);	
	       P2 = getRectPoints(p2,p1,wd,wd+1);	
	  }

	  int xPoints[] = {P1[0].x,P2[0].x,P2[1].x,P1[1].x};
	  int yPoints[] = {P1[0].y,P2[0].y,P2[1].y,P1[1].y};
	  g.fillPolygon(xPoints,yPoints,4);
     }

/******************************************************************************
**
**     Method name:             drawSLine (class GeoPlot)
**
**     Purpose:                 This method draws all the different types of 
**                              lines except the lines having the same source
**                              and destination ie. circles.
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the  
**                                           applet is running in.
**                              int lineSize - width of the line
**                              Point p[] - array of points that make up the 
**                                          different parts of the line 
**                              direc     - direction of arrow 
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public static void drawSLine(Graphics g,int lineSize, 
				                        Point p[], int direc) 
     {
	  Point P1[] = new Point[2];
	  Point P2[] = new Point[2];
	  Point P3[] = new Point[2];

	  if( lineSize == 1)
	  {
	       switch(p.length)
	       {
	            case 2:
			   g.drawLine(p[0].x,p[0].y,p[1].x,p[1].y);
			   break;
	            case 4:
	            case 6:
			   for (int i=0; i<p.length-1; i++)
			   {
			        if (!(p[i].x == r.width && p[i+1].x == 0))
				{
				     g.drawLine(p[i].x,p[i].y,p[i+1].x,
						                    p[i+1].y);
				}
			   }
			   break;
	       }
	  }
	  else
	  {
	       switch(p.length)
	       {
	            case 2:
			   drawThickLine(g, lineSize, p[0],p[1]);
			   break;
	            case 4:
			   if ((p[1].x == r.width && p[2].x == 0))
			   {
			        drawThickLine(g, lineSize, p[0], p[1]);
				drawThickLine(g, lineSize, p[2], p[3]);
			   }
			   else
			   {
			        g.drawLine(p[0].x,p[0].y,p[1].x,p[1].y);
			        drawThickLine(g, lineSize, p[1], p[2]);
				g.drawLine(p[2].x,p[2].y,p[3].x,p[3].y);
			   }
			   break;
	            case 6:
			   g.drawLine(p[0].x,p[0].y,p[1].x,p[1].y);
			   drawThickLine(g, lineSize, p[1], p[2]);
			   drawThickLine(g, lineSize, p[3], p[4]);
			   g.drawLine(p[4].x,p[4].y,p[5].x,p[5].y);
			   break;
	       }
	  }

	  int wd = lineSize / 2;

	  if (lineSize > 1)
	  {
	       if (lineSize < 4)
	       {
		    lineSize = 4;
	       }
		   
	       lineSize /= 2;

	       switch(p.length)
	       {
	            case 4:
			   if ((lineSize % 2) == 0)
			   {
			        P1 = getRectPoints(p[0],p[1],wd,wd);	
				P2 = getRectPoints(p[1],p[0],wd,wd);
			   }
			   else
			   {
			        P1 = getRectPoints(p[0],p[1],wd,wd+1);	
				P2 = getRectPoints(p[1],p[0],wd,wd+1);	
			   }

			   if (direc == 5 || direc == 6)
			   {
			       drawArrow(g, lineSize, P1[0],P2[0],0);
			       drawArrow(g, lineSize, P1[1],P2[1],1);
			   }
	            case 2:
			   if ((lineSize % 2) == 0)
			   {
			        P1 = getRectPoints(p[p.length-1],
						   p[p.length-2],wd,wd);
				P2 = getRectPoints(p[p.length-2],
						   p[p.length-1],wd,wd);
			   }
			   else
			   {
			        P1 = getRectPoints(p[p.length-1],
						   p[p.length-2],wd,wd+1);
				P2 = getRectPoints(p[p.length-2],
						   p[p.length-1],wd,wd+1);
			   }

			   if (direc == 4 || direc == 6)
			   {
			       drawArrow(g, lineSize, P1[0],P2[0],0);
			       drawArrow(g, lineSize, P1[1],P2[1],1);
			   }

			   if ((direc == 5 || direc == 6) && p.length == 2)
			   {
			       drawArrow(g, lineSize, P2[0],P1[0],0);
			       drawArrow(g, lineSize, P2[1],P1[1],1);
			   }
	       }
	  }

	  if (direc == 1 || direc == 3)
	  {
	       P1 = getArrowPoints(p[p.length-1],p[p.length-2],7,45);
	       drawArrow(g, 1, p[p.length-1],P1[0],0);
	       drawArrow(g, 1, p[p.length-1],P1[1],0);
	  }

	  if (direc == 2 || direc == 3)
	  {
	       P1 = getArrowPoints(p[0],p[1],7,45);
	       drawArrow(g, 1, p[0], P1[0],0);
	       drawArrow(g, 1, p[0], P1[1],0);
	  }
     }

/******************************************************************************
**
**     Method name:             drawSCircle (class GeoPlot)
**
**     Purpose:                 Draws a circle for linesize = 1 and draws 
**                              linesize number of circles for linesize > 1,
**                              reducing the radius by one each time.
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the
**                                           applet is running in.
**                              int lineSize - width of the line
**                              Point p[] - the center of the circle and also
**                                          radius stored in p[1].x 
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public static void drawSCircle(Graphics g,int lineSize,Point p[]) 
     {
	  if( lineSize == 1)
	  {
	       g.drawOval(p[0].x-p[1].x,p[0].y-p[1].x,2*p[1].x,2*p[1].x);
	  }
	  else
	  {
	       int r = p[1].x;
	       for (int i=0; i<lineSize; i++)
	       {
		    g.drawOval(p[0].x-r,p[0].y-r,2*r,2*r--);
		    if (r == 4)
		    {
			break;
		    }
	       }
	  }
     }

/******************************************************************************
**
**     Method name:             drawSelfLine (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the  
**                                           applet is running in.
**                              Line line[]
**                              float latp1
**                              float lonp1
**                              Node nodeP1
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private void drawSelfLine(Graphics g, Line line[], float latp1, 
			                              float lonp1, Node nodeP1)
     {
	  int lineSize;
	  int direc;
          int numofCkeys = color_key.capacity();
	  int numofSkeys = size_key.capacity();
	  int angleFac = 0;
	  int lineNum=0;
	  int quad = 1;
	  int radius;
	  int X;
	  int Y;
	  String lineTitle;

	  Point p1 = latLonToPoint(latp1,lonp1);
	  
	  int max_radius = Math.min(r.width,r.height)/20;
	  if (max_radius > 25)
	  {
	      max_radius = 25;
	  }
	  
	  int xtmp = Math.min(Math.abs(p1.x-r.width),p1.x);
	  int ytmp = Math.min(Math.abs(p1.y-r.height),p1.y);
	  
	  if (xtmp > max_radius && ytmp > max_radius)
	  {
	       radius = max_radius;
	  }
	  else
	  {
	       radius = Math.min(xtmp,ytmp);
	  }

	  for (int i=1; i<25; i++)
	  {
	       Point ptmp[] = new Point[2];
	       ptmp[1] = new Point(0,0);
	       ptmp[1].x = radius;

	       if (i > 12)
	       {
		    ptmp[0] = getArcCenter(p1, angleFac, ptmp[1].x, 1-quad);
	       }
	       else
	       {
		    ptmp[0] = getArcCenter(p1, angleFac, ptmp[1].x, 2-quad);
	       }


	       if ( (i % 20) == 0)
	       {
		    angleFac = 0;
	       }

	       if (!(r.contains(new Point(ptmp[0].x-ptmp[1].x,
					  ptmp[0].y-ptmp[1].x)) 
		     && r.contains(new Point(ptmp[0].x+ptmp[1].x,
					     ptmp[0].y+ptmp[1].x))))
	       {
		    quad *= -1;
		    
		    if ( (i % 2) == 0) 
		    {
			 angleFac += 15;
		    }
		    continue;
	       }
	       
	       setColor(g, line[lineNum],numofCkeys);
	       lineSize = getNodeLineSize(line[lineNum],numofSkeys);
	       line[lineNum].setSize(lineSize);

	       drawSCircle(g,lineSize,ptmp);

	       if ((lineTitle = line[lineNum].getTitle()) != "")
	       {
		    int position=2;
		    if (lineTitle.indexOf(',') > 0)
		    {
			 StringTokenizer tokenizer = new StringTokenizer
			                                       (lineTitle,",");
			 while (tokenizer.countTokens() > 1)
			 {
			     tokenizer.nextToken();
			 }

			 try
			 {
			      position = Integer.parseInt
				                       (tokenizer.nextToken());
			 }
			 catch(NumberFormatException nfe) 
			 { 
			      lineTitle = lineTitle.concat(",2");
			 }
		    }
		    else
		    {
			 lineTitle = lineTitle.concat(",2");
		    }

                    
		    X = ptmp[0].x-ptmp[1].x;
		    Y = ptmp[0].y-ptmp[1].x;
		    
		    g.setFont(new Font("Dialog", Font.BOLD,fontSize));
		    fm = g.getFontMetrics();

		    switch(position)
		    {
		         case 1:
			        drawLineTitle(g,1,new Point(X,Y),
					      new Point(X,Y+(2*ptmp[1].x)),
					      lineTitle);
				break;
		         case 2:
				Y -= fm.getMaxAdvance();
 			        drawLineTitle(g,1,new Point(X,Y),
					      new Point(X+(2*ptmp[1].x),Y),
					      lineTitle);
				break;
		         case 3:
			        drawLineTitle(g,1,new Point(X+(2*ptmp[1].x),Y),
					      new Point(X+(2*ptmp[1].x),
							Y+(2*ptmp[1].x)),
					      lineTitle);
				break;
		         case 4:
			        Y = ptmp[0].y + ptmp[1].x + 
				                      (2*fm.getMaxAdvance()); 
				drawLineTitle(g,1,new Point(X,Y),
					      new Point(X+(2*ptmp[1].x),Y),
					      lineTitle);
		    }
	       }


	       line[lineNum].setType(8);
	       line[lineNum].setPoint(ptmp,8);

	       nodeP1.quad_matrix[getQuadrant(p1,ptmp[0])]++;

	       if (++lineNum == line.length)
	       {
		    break;
	       }

	       quad *= -1;
	       if ( (i % 2) == 0) 
	       {
		    angleFac += 15;
	       }
	  }
     }

/******************************************************************************
**
**     Method name:             getArcCenter (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Point p
**                              int angle
**                              int hyp
**                              int quad
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private Point getArcCenter(Point p, int angle, int hyp, int quad)
     {
	  Point pt = new Point();

	  float anglerad = new Double(angle * Math.PI / 180.0).floatValue();

	  int x = new Double(Math.cos(anglerad) * hyp).intValue();
	  int y = new Double(Math.sin(anglerad) * hyp).intValue();

	  switch(quad)
	  {
	       case quadone:
		               pt.x = p.x+x;
			       pt.y = p.y-y;
		               break;
 	       case quadtwo: 
		               pt.x = p.x-y;
			       pt.y = p.y-x;
		               break;
	       case quadthree:
		               pt.x = p.x-x;
			       pt.y = p.y+y;
		               break;
	       case quadfour:
		               pt.x = p.x+y;
			       pt.y = p.y+x;
	  }

	  return pt;
     }

/******************************************************************************
**
**     Method name:             drawLineTitle (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Graphics g - the graphics context describing the
**                                           graphics environment the applet is 
**                                           running in. 
**                              int lineSize - width of the line 
**                              Point p1 - start point of the line 
**                              Point p2 - end point of the line
**                              String lineTitle
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private void drawLineTitle(Graphics g, int lineSize, Point p1, Point p2, 
				                              String lineTitle)
     {
	  String tmp = new String();

	  g.setFont(new Font("Dialog", Font.BOLD,fontSize));
	  fm = g.getFontMetrics();
	  int position=3;
	  if (lineTitle.indexOf(',') > 0)
	  {
	       StringTokenizer tokenizer = new StringTokenizer(lineTitle,",");
	       String temp = new String();

	       while (tokenizer.countTokens() > 2)
	       {
		    temp = temp.concat(tokenizer.nextToken());
		    temp = temp.concat(",");
	       }
	       temp = temp.concat(tokenizer.nextToken());

	       try
	       {
		    tmp = tokenizer.nextToken();
		    position = Integer.parseInt(tmp);
	       }
	       catch(NumberFormatException nfe) 
	       { 
		   temp = temp.concat(",");
		   temp = temp.concat(tmp);
	       }
	       lineTitle = temp;
	  }

	  Point p = new Point();

	  p.x = Math.min(p1.x,p2.x) + Math.abs(p1.x - p2.x)/2;
	  p.y = Math.min(p1.y,p2.y) + Math.abs(p1.y - p2.y)/2;

	  int strlen = fm.stringWidth(lineTitle);

	  if ( !(p1.x == p2.x || p1.y == p2.y) )
	  {
	       switch(getQuadP1(p1,p2))
	       {
	            case quadone:
	            case quadthree:
		                switch(position)
				{
				     case 1:
					    p.x = p.x - (lineSize+1)/2 - 
						  strlen - fm.getMaxAdvance();
					    p.y = p.y - fm.getLeading() - 
						  fm.getDescent();
					    break;
				     case 2:
					    p.x = p.x - (lineSize+1)/2 - 
						  strlen/2;
					    break;
				     case 3:
					    p.x = p.x + (lineSize+1)/2 + 
						  fm.getMaxAdvance();
					    p.y = p.y + fm.getLeading() + 
						  fm.getAscent();
				}
		                break;
	            case quadtwo:
	            case quadfour:
		                switch(position)
				{
				     case 3:
					    p.x = p.x - (lineSize+1)/2 - 
						  strlen - fm.getMaxAdvance();
					    p.y = p.y + fm.getLeading() + 
						  fm.getAscent();
					    break;
				     case 2:
					    p.x = p.x - (lineSize+1)/2 - 
						  strlen/2;
					    p.y += fm.getAscent();
					    break;
				     case 1:
					    p.x = p.x + (lineSize+1)/2 + 
						  fm.getMaxAdvance();
					    p.y = p.y - fm.getLeading() - 
						  fm.getDescent();
				}
	       }
	  }
	  else
	  {
	       if (p1.x == p2.x)
	       {
		    switch(position)
		    {
		         case 1:
			        p.x = p.x - (lineSize+1)/2 - strlen - 
				      fm.getMaxAdvance();
				break;
		         case 2:
			        p.x = p.x - (lineSize+1)/2 - strlen/2;
				break;
		         case 3:
			        p.x = p.x + (lineSize+1)/2 + 
				      fm.getMaxAdvance();
		    }
	       }
	       else
	       {
		    p.x = p.x - strlen/2;
		    switch(position)
		    {
		         case 1:
				p.y = p.y - fm.getLeading() - (lineSize+1)/2;
				break;
		         case 3:
			        p.y += fm.getAscent() + (lineSize+1)/2;
				break;
		         case 2:
				p.y = p.y + fm.getLeading() + fm.getDescent();
		    }
	       }
	  }

	  // Handle case when p.x and p.y become invalid values
	  g.drawString(lineTitle,p.x, p.y);			 
     }

/******************************************************************************
**
**     Method name:             drawLine (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the  
**                                           applet is running in.
**                              Line line[] - all the lines that exist between 
**                                            the first and second points.
**                              float latp1 - latitude of first point
**                              float lonp1 - longitude of first point
**                              float latp2 - latitude of second point
**                              float lonp2 - longitude of second point
**                              Node nodeP1 - 
**                              Node nodeP2
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private void drawLine(Graphics g, Line line[], float latp1, float lonp1, 
			   float latp2, float lonp2, Node nodeP1, Node nodeP2)
     {
	  int lineSize;
	  int prevMaxLineSize=1;
	  int direc;
          int numofCkeys = color_key.capacity();
	  int numofSkeys = size_key.capacity();
	  String lineTitle = new String();

	  Point p1 = latLonToPoint(latp1,lonp1);
	  Point p2 = latLonToPoint(latp2,lonp2);
          Point P1[] = new Point[2];
          Point P2[] = new Point[2];

	  if (comparelon(lonp1,lonp2) < Math.abs(bot_lon-top_lon)/2 )
	  {
	       int lineSize1;
	       int lineSize2;
	       int start = 0;
	       boolean extra = false;
	       int w=0;

		    
	       if ((line.length % 2) != 0)
	       {
		    setColor(g, line[0],numofCkeys);
		    lineSize = getNodeLineSize(line[0],numofSkeys);
		    line[0].setSize(lineSize);
		    
		    if (lineSize > 1)
		    {
			 w = lineSize/2;
			 direc = line[0].setDirec(1);
		    }
		    else
		    {
			 direc = line[0].setDirec(0);
		    }


		    P1[0] = p1;
		    P1[1] = p2;

		    drawSLine(g, lineSize, P1, direc);	

		    if ((lineTitle = line[0].getTitle()) != "")
		    {
			 drawLineTitle(g,lineSize,p1,p2,lineTitle);
		    }

		    line[0].setType(1);
		    line[0].setPoint(p1,p2);

		    nodeP1.quad_matrix[getQuadrant(p1,p2)]++;
		    nodeP2.quad_matrix[getQuadrant(p2,p1)]++;

		    start = 1;
	       }

	       float angle = 45;

	       for (int i=start; i<line.length; i+=2)
	       {
		    Point ptmp1[] = new Point[4];
		    Point ptmp2[] = new Point[4];

		    lineSize1 = getNodeLineSize(line[i],numofSkeys);

		    lineSize2 = getNodeLineSize(line[i+1],numofSkeys);
		    line[i].setSize(lineSize1);
		    line[i+1].setSize(lineSize2);
		    
		    if (lineSize1 == 1 && lineSize2 == 1)
		    {    
			 w += 5;
			 if (extra)
			 {
			      w += prevMaxLineSize/2;
			      extra = false;
			 }
		    }
		    else if (lineSize1 >= lineSize2)
		    {
			 w += lineSize1;
			 prevMaxLineSize = lineSize1;
			 extra = true; 
		    }
		    else
		    {
			 w += lineSize2;
			 prevMaxLineSize = lineSize2;
			 extra = true;
		    }

		    if (angle > 165)  // xxx - check 165
		    {
			 continue;
		    }

		    P1 = getArrowPoints(p1,p2,w,
					       new Double(angle).floatValue());
		    P2 = getArrowPoints(p2,p1,w,
					       new Double(angle).floatValue());
			 
		    setColor(g, line[i],numofCkeys);
		   
		    direc = line[i].setDirec(0);

		    if (((Node)nodes.get(line[i].getSrc())).getNodNum() 
							 == nodeP1.getNodNum())
		    {
			ptmp1[0] = p1;
			ptmp1[1] = P1[0];
			ptmp1[2] = P2[0];
			ptmp1[3] = p2;
		    }
		    else if (((Node)nodes.get(line[i].getSrc())).getNodNum() 
							 == nodeP2.getNodNum())
		    {
			ptmp1[0] = p2;
			ptmp1[1] = P2[0];
			ptmp1[2] = P1[0];
			ptmp1[3] = p1;
		    }

		    drawSLine(g, lineSize1, ptmp1, direc);

		    if ((lineTitle = line[i].getTitle()) != "")
		    {
			 drawLineTitle(g,lineSize1,P1[0],P2[0],lineTitle);
		    }

		    line[i].setType(3);
		    line[i].setPoint(ptmp1,0);

		    nodeP1.quad_matrix[getQuadrant(p1,P1[0])]++;
		    nodeP2.quad_matrix[getQuadrant(p2,P2[0])]++;
		    
		    setColor(g, line[i+1],numofCkeys);
		    
		    direc = line[i+1].setDirec(0);

		    if (((Node)nodes.get(line[i+1].getSrc())).getNodNum() 
			                                  == nodeP1.getNodNum())
		    {
			ptmp2[0] = p1;
			ptmp2[1] = P1[1];
			ptmp2[2] = P2[1];
			ptmp2[3] = p2;
		    }
		    else if (((Node)nodes.get(line[i+1].getSrc())).getNodNum() 
			                                   == nodeP2.getNodNum())
		    {
			ptmp2[0] = p2;
			ptmp2[1] = P2[1];
			ptmp2[2] = P1[1];
			ptmp2[3] = p1;
		    }		    
		    

		    drawSLine(g, lineSize2, ptmp2, direc);

		    if ((lineTitle = line[i+1].getTitle()) != "")
		    {
			 drawLineTitle(g,lineSize2,P1[1],P2[1],lineTitle);
		    }

		    line[i+1].setType(3);
		    line[i+1].setPoint(ptmp2,0);

		    nodeP1.quad_matrix[getQuadrant(p1,P1[1])]++;
		    nodeP2.quad_matrix[getQuadrant(p2,P2[1])]++;
			 
		    angle += 15;
	       }
	  }
	  else
	  {
	       if (pointWithinComponent(p1) && pointWithinComponent(p2))
	       {
		    Point P[] = new Point[2];
		    P[0] = new Point(0,0);
		    boolean flipped = false;

		    if (p1.x < p2.x)
		    {
			 Point pTemp = p2;
			 p2 = p1;
			 p1 = pTemp;

			 Node nTemp = nodeP2;
			 nodeP2 = nodeP1;
			 nodeP1 = nTemp;

			 flipped = true;
		    }

		    P[0] = new Point(p1.x - r.width,p1.y);
		    P[1] = p2;

		    float m = new Float(P[0].y - P[1].y).floatValue()/
			      new Float(P[0].x - P[1].x).floatValue();
		    float c = new Integer(P[0].y).floatValue() - 
			      (new Float(m).floatValue() * 
			       new Integer(P[0].x).floatValue());

		    int y = Math.round(c);

		    int lineSize1;
		    int lineSize2;
		    int start = 0;
		    boolean extra = false;
		    int w=0;		
		    int h=0;
		    
		    if ((line.length % 2) != 0)
		    {
			 setColor(g, line[0],numofCkeys);
			 lineSize = getNodeLineSize(line[0],numofSkeys);
			 line[0].setSize(lineSize);
			      
			 Point ptmp[] = new Point[4];
			 ptmp[0] = p1;
			 ptmp[1] = new Point(r.width,y);
			 ptmp[2] = new Point(0,y);
			 ptmp[3] = p2;	

			 if (flipped)
			 {
			      switch(line[0].getDirec())
			      {
			           case 1:
				          line[0].chngDirec(2);
					  break;
			           case 2:
				          line[0].chngDirec(1);
			      }
			 }

			 if (lineSize > 1)
			 {
			      w = lineSize/2;
			      direc = line[0].setDirec(1);
			 }
			 else
			 {
			      direc = line[0].setDirec(0);
			 }

			 drawSLine(g, lineSize,ptmp, direc);

			 if ((lineTitle = line[0].getTitle()) != "")
			 {
			      if ((r.width - p1.x) > p2.x)
			      {
				   drawLineTitle(g,lineSize,ptmp[0],ptmp[1],
						 lineTitle);
			      }
			      else
			      {
				   drawLineTitle(g,lineSize,ptmp[2],ptmp[3],
						 lineTitle);
			      }
			 }

			 line[0].setType(3);
			 line[0].setPoint(ptmp,1);			      

			 nodeP1.quad_matrix[getQuadrant(p1,
					             new Point(r.width,y))]++;
			 nodeP2.quad_matrix[getQuadrant(new Point(0,y),p2)]++;

			 start = 1;
		    }
			 
		    float angle = 45;
		    
		    for (int i=start; i<line.length; i+=2)
		    {
			 Point ptmp1[] = new Point[6];
			 Point ptmp2[] = new Point[6];
			 
			 lineSize1 = getNodeLineSize(line[i],numofSkeys);
			 lineSize2 = getNodeLineSize(line[i+1],numofSkeys);
			 line[i].setSize(lineSize1);
			 line[i+1].setSize(lineSize2);
			 
			 if (lineSize1 == 1 && lineSize2 == 1)
			 {    
			      w += 5;
			      if (extra)
			      {
				   w += prevMaxLineSize/2;;
				   extra = false;
			      }
			 }
			 else if (lineSize1 >= lineSize2)
			 {
			      w += lineSize1;
			      prevMaxLineSize = lineSize1;
			      extra = true; 
			 }
			 else
			 {
			      w += lineSize2;
			      prevMaxLineSize = lineSize2;
			      extra = true;
			 }
			 

			 if (angle > 165)  // xxx - check 165
			 {
			      continue;
			 }

			 // Code to draw Stuff for the LHS

			 P1 = getArrowPoints(p1,new Point(r.width,y),w,angle);
			 h = w;

			 if (flipped)
			 {
			      switch(line[i].getDirec())
			      {
			           case 1:
				          line[i].chngDirec(2);
					  break;
			           case 2:
				          line[i].chngDirec(1);
			      }
			 }
			 line[i].setDirec(0);

			 if (p1.y < y)
			 {
			      h = -w;
			 }
     
			 ptmp1[0] = p1;
			 ptmp1[1] = P1[0];
			 ptmp1[2] = new Point(r.width,y - h);

			 if (flipped)
			 {
			      switch(line[i+1].getDirec())
			      {
			           case 1:
				          line[i+1].chngDirec(2);
					  break;
			           case 2:
				          line[i+1].chngDirec(1);
			      }
			 }
			 line[i+1].setDirec(0);

			 ptmp2[0] = p1;
			 ptmp2[1] = P1[1];
			 ptmp2[2] = new Point(r.width,y + h);	
			 
			 nodeP1.quad_matrix[getQuadrant(p1,P1[0])]++;
			 nodeP1.quad_matrix[getQuadrant(p1,P1[1])]++;
			      

			 // Code to draw stuff on the RHS

			 P1 = getArrowPoints(p2,new Point(0,y),w,angle);

			 if (p2.y > y)
			 {
			      h = -w;
			 }

			 ptmp1[3] = new Point(0,y - h);
			 ptmp1[4] = P1[0];
			 ptmp1[5] = p2;
	   
			 ptmp2[3] = new Point(0,y + h);
			 ptmp2[4] = P1[1];
			 ptmp2[5] = p2;			      

			 line[i].setType(5);
			 line[i].setPoint(ptmp1,1);
			 line[i+1].setType(5);
			 line[i+1].setPoint(ptmp2,1);

			 setColor(g, line[i],numofCkeys);
			 direc = line[i].getDirec();
			 drawSLine(g, lineSize1, ptmp1, direc);

			 if ((lineTitle = line[i].getTitle()) != "")
			 {
			      if ((r.width - ptmp1[1].x) > ptmp1[4].x)
			      {
				   drawLineTitle(g,lineSize1,ptmp1[1],
						 ptmp1[2],lineTitle);
			      }
			      else
			      {
				   drawLineTitle(g,lineSize1,ptmp1[3],
						 ptmp1[4],lineTitle);
			      }
			 }

			 setColor(g, line[i+1],numofCkeys);
			 direc = line[i+1].getDirec();
			 drawSLine(g, lineSize2, ptmp2, direc);

			 if ((lineTitle = line[i+1].getTitle()) != "")
			 {
			      if ((r.width - ptmp2[1].x) > ptmp2[4].x)
			      {
				   drawLineTitle(g,lineSize2,ptmp2[1],
						 ptmp2[2],lineTitle);
			      }
			      else
			      {
				   drawLineTitle(g,lineSize2,ptmp2[3],
						 ptmp2[4],lineTitle);
			      }
			 }

			 nodeP2.quad_matrix[getQuadrant(p2,P1[0])]++;
			 nodeP2.quad_matrix[getQuadrant(p2,P1[1])]++;

			 // xxx - check angle increment
			 angle += 15;
		    }
	       }     

	       /*The Code below which is not complete at this time, is to 
		 handle cases where one of the end points don't fall
		 within the region of the image
	       */

	       else if (!pointWithinComponent(p1))
	       {
		    // From P2's perspective of things 
		    Point orgp2 = latLonToPoint(latp2,lonp2);
		    float x1 = new Float(r.width - orgp2.x).floatValue();
		    float y1;
		    float laty;
		    if (latp2 < latp1)
		    {
			 y1 = new Float(orgp2.y).floatValue();
			 laty = top_lat - latp2;
		    }
		    else
		    {
			 y1 = new Float(r.height - orgp2.y).floatValue();
			 laty = Math.abs(bot_lat - latp2);
		    }
		    
		    float latx = bot_lon - lonp2;
		    
		    float realx = difflon(lonp1,lonp2)*x1 /latx;
		    float realy = difflat(latp1,latp2)*y1 /laty;
		    
		    float slope = realx / realy;
		    int y1dash = new Float((r.width - p2.x) / 
					            slope).intValue();
		    
		    if (latp2 < latp1)
		    {
			 g.drawLine(r.width,p2.y-y1dash,p2.x,p2.y);

			 nodeP2.quad_matrix[getQuadrant(p2,
					    new Point(r.width,p2.y-y1dash))]++;
		    }
		    else
		    {
			 g.drawLine(r.width,p2.y+y1dash,p2.x,p2.y);
			 
			 nodeP2.quad_matrix[getQuadrant(p2,
					    new Point(r.width,p2.y+y1dash))]++;
		    }
	       }

	       else
	       {
		    // From P1's perspective of things 
		    Point orgp1 = latLonToPoint(latp1,lonp1);
		    float x1 = new Float(orgp1.x).floatValue();
		    float y1;
		    float laty;
		    if (latp1 > latp2)
		    {
			 y1 = new Float(r.height - orgp1.y).floatValue();
			 laty = Math.abs(bot_lat - latp1);
		    }
		    else
		    {
			 y1 = new Float(orgp1.y).floatValue();
			 laty = latp1 - top_lat;
		    }
		    
		    float latx = lonp1 - top_lon;
			      
		    float realx = difflon(lonp1,lonp2)*x1 /latx;
		    float realy = difflat(latp1,latp2)*y1 /laty;
		    
		    float slope = realx / realy;
		    int y1dash = new Float(p1.x / slope).intValue();
		    
		    if (latp1 > latp2)
		    {
			 g.drawLine(p1.x,p1.y,1,p1.y+y1dash);
			 
			 nodeP1.quad_matrix[getQuadrant(p1,
						  new Point(1,p1.y+y1dash))]++;
		    }
		    else
		    {
			 g.drawLine(p1.x,p1.y,1,p1.y-y1dash);
			 
			 nodeP1.quad_matrix[getQuadrant(p1,
						  new Point(1,p1.y-y1dash))]++;
		    }
	       }
	  }
     }


/******************************************************************************
**
**     Method name:             paintData (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the  
**                                           applet is running in.
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void paintData(Graphics g)
     {
	  String token = new String();
	  String nodTitle;
	  String temp;
	  StringTokenizer tokenizer;
	  int numofLines;
	  int numofNodes;
	  int numofCkeys;
	  int numofSkeys;
	  int lineSize;
	  LineMatrix lmn1;
	  LineMatrix lmn2;

	  Line line;
	  Node nodeP1;
	  Node nodeP2;
	  
	  float latp1 = 0;
	  float lonp1 = 0;
	  float latp2 = 0;
	  float lonp2 = 0;
	  Point p1 = new Point();
	  Point p2 = new Point();

	  paintDataCalled=true;
	  g.drawImage(map, 0,0,r.width, r.height, null);
	  g.setFont(new Font("Dialog", Font.BOLD,fontSize));
	  fm = g.getFontMetrics();
	  g.setColor(fgcolor);
	  g.drawString(title,(r.width - fm.stringWidth(title))/2, fontSize);
	  g.drawString(date,r.width-fm.stringWidth(date)-10, fontSize);
	  
	  color_key.trimToSize();
	  size_key.trimToSize();
	  lines.trimToSize();
	  numofLines = lines.capacity();
	  numofCkeys = color_key.capacity();
	  numofSkeys = size_key.capacity();
	  
	  for ( int i=0; i < numofLines; i++)
	  {
	       line = (Line)lines.elementAt(i);

	       nodeP1 = (Node)nodes.get(line.getSrc());

	       if ( nodeP1 == null)
	       {
		    continue;
	       }
	       else
	       {
		    latp1 = nodeP1.getLat();
		    lonp1 = nodeP1.getLon();
		    lmn1 = (LineMatrix)lineMatrix.get(
						line.getSrc()+line.getDest());
		    p1 = latLonToPoint(latp1,lonp1);
	       }
	       
	       nodeP2 = (Node)nodes.get(line.getDest());

	       if ( nodeP2 == null)
	       {
		    continue;
	       }
	       else
	       {		
		    latp2 = nodeP2.getLat();
		    lonp2 = nodeP2.getLon();
		    lmn2 = (LineMatrix)lineMatrix.get(
						 line.getDest()+line.getSrc());
		    p2 = latLonToPoint(latp2,lonp2);
	       }

	       if ( lmn1 == lmn2 && lmn1.getValue() > 0)
	       {
		    // case where node has a line going to itself
		    int nlin = lmn1.getValue();
		    int nlincount = 0;
		    Line mline[] = new Line[nlin];
		    
		    String src = line.getSrc();

		    if (lmn1.getValue() == 1)
		    {
			 mline[0] = line;
			 lmn1.decrValue();
		    }
		    else
		    {
			 for (int k=0; k< numofLines; k++)
			 {
			      line = (Line)lines.elementAt(k);

			      if (line.getSrc().equals(src) 
				  && line.getDest().equals(src))
			      {
				   mline[nlincount++] = line;
				   lmn1.decrValue();
			      }
			 }
		    }			

		    drawSelfLine(g, mline, latp1, lonp1, nodeP1);
	       }
	       else if (lmn1.getValue() == 1 && lmn2 == null)
	       {
		    // case where there is only one line between nodes i and j
		    Line mline[] = new Line[1];
		    mline[0] = line;
		    lmn1.decrValue();
		    drawLine(g, mline, latp1, lonp1, latp2, lonp2, 
			                                      nodeP1, nodeP2);
	       }
	       else 
	       {
		    /* case where there is more that one line between 
		       nodes i and j
		    */
		    int nlin;

		    if (lmn1 != null && lmn2 != null)
		    {
			 nlin = lmn1.getValue() + lmn2.getValue();
		    }
		    else if (lmn2 == null)
		    {
			 nlin = lmn1.getValue();
		    }
		    else
		    {
			 nlin = lmn2.getValue();
		    }
		    
                    if ( nlin <= 0)
		    {
			 continue;
		    }
 
		    int nlincount = 0;
		    Line mline[] = new Line[nlin];
		    
		    String src = line.getSrc();
		    String dest = line.getDest();

		    for (int k=0; k< numofLines; k++)
		    {
			 line = (Line)lines.elementAt(k);

			 if (line.getSrc().equals(src) 
			     && line.getDest().equals(dest))
			 {
			      mline[nlincount++] = line;
			      lmn1.decrValue();
			 }
			 else if (line.getSrc().equals(dest) 
				  && line.getDest().equals(src))
			 {
			      mline[nlincount++] = line;
			      lmn2.decrValue();
			 }
		    }			

		    drawLine(g, mline, latp1, lonp1, latp2, lonp2, 
			                                      nodeP1, nodeP2);
	       }
	  }
	  

	  Enumeration enode = nodes.elements();
	  while(enode.hasMoreElements())
	  {
	       nodTitle="";

	       nodeP1 = (Node)enode.nextElement();
	      
	       latp1 = nodeP1.getLat();
	       lonp1 = nodeP1.getLon();
	       p1 = latLonToPoint(latp1,lonp1);		    
		    
	       int width = getNodeLineSize(nodeP1,numofSkeys);
	       nodeP1.setSize(width);
	       setColor(g, nodeP1, numofCkeys);

	       nodeP1.setPoint(p1);

	       g.fillOval(p1.x-(width/2),p1.y-(width/2),width,width);

	       if ((nodTitle = nodeP1.getTitle()) != "")
	       {
		    g.setColor(fgcolor);
		    int quad=56;
		    if (nodTitle.indexOf(',') > 0)
		    {
			 tokenizer = new StringTokenizer(nodTitle,",");
			 temp = new String();

			 while (tokenizer.countTokens() > 2)
		         {
			      temp = temp.concat(tokenizer.nextToken());
			      temp = temp.concat(",");
			 }
			 temp = temp.concat(tokenizer.nextToken());

			 try
			 {
			      token = tokenizer.nextToken();
			      quad = Integer.parseInt(token);
			 }
			 catch(NumberFormatException nfe) 
			 { 
			      temp = temp.concat(",");
			      temp = temp.concat(token);
			 }
			 nodTitle = temp;
		    }
		    else
		    {
			 quad = nodeP1.getQuadrant();
		    }
		    Point titlepoint = getTitleXY(nodTitle,fm,quad,p1,width);
		    g.drawString(nodTitle,titlepoint.x, titlepoint.y);			 
	       }
	  }

	  
	  // Display the color key

	  if (colorkey && !hide_key)
	  {
	       int curX=Ckey_x;
	       int curY;

	       g.setColor(Color.white);
	       g.fillRect(Ckey_x,Ckey_y,Ckey_width,Ckey_height);
		   
	       g.setColor(Color.black);
	       g.setFont(new Font("monospaced", Font.PLAIN,Ckey_fontSize));
	       fm = g.getFontMetrics();
	       curY = Ckey_y+fm.getLeading()+fm.getAscent();
	       g.drawString(Ckey_title,curX + (Ckey_width - 
				      fm.stringWidth(Ckey_title)) / 2, curY);
	       curY = curY + fm.getAscent();
	       
	       g.setFont(new Font("monospaced", Font.PLAIN,Ckey_fontSize-KEY_DATA_SIZE));
	       fm = g.getFontMetrics();
	       
	       for(int ck=0 ; ck < numofCkeys; ck++, curY += fm.getHeight())
	       {
		    key = (Key)color_key.elementAt(ck);		   
		    setColor(g,key.getValue());

		    g.fillRect(curX+2, curY-fm.getAscent(), 10, 
			                                      fm.getAscent()); 
		    g.setColor(Color.black);
		    g.drawString(key.toString(), curX+15, curY);
	       }
	  }


	  // Display the size key
	  
	  if (sizekey && !hide_key)
	  {
	       Font font;
	       String label;
	       int labelWidth;
	       int height;
	       int prevHeight;
	       int curX=Skey_x;
	       int curY;
	       int curH=0;

	       g.setColor(Color.white);
	       g.fillRect(Skey_x,Skey_y,Skey_width,Skey_height);
		   
	       g.setColor(Color.black);
	       g.setFont(new Font("monospaced", Font.PLAIN,Skey_fontSize));
	       fm = g.getFontMetrics();
	       curY = Skey_y+fm.getLeading()+fm.getAscent();
	       g.drawString(Skey_title,curX + (Skey_width - 
				        fm.stringWidth(Skey_title)) / 2, curY);
	       curY = curY + 2*fm.getAscent();
	       curX += 5;

	       g.setFont(new Font("monospaced", Font.PLAIN,Skey_fontSize-KEY_DATA_SIZE));
	       g.setColor(Color.black);
	       fm = g.getFontMetrics();
	       height = prevHeight = 0;
	       
	       for(int sk=0 ; sk < numofSkeys; sk++)
	       {
		    key = (Key)size_key.elementAt(sk);		   
		    label = key.toString();
		    labelWidth = fm.stringWidth(label);
		    g.drawString(label, curX + labelWidth/4, curY);
		    
		    try
		    {
			 prevHeight = height;
			 height = Integer.parseInt(key.getValue());
			 if (sk == 0)
			 {
			      prevHeight = height;
			 }
			 
		    }
		    catch(NumberFormatException nfe)
		    {
			height = 0;
		    }
		    
		    curH += (prevHeight - height)/2; 
		    g.fillRect(curX, curY+fm.getHeight()+curH, 
			                              3*labelWidth/2,height); 
		    curX += 3*labelWidth/2;
	       }  

	  }
	  
	  repaint();
     }
    

/******************************************************************************
**
**     Method name:             getTitleXY (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              String title
**                              FontMetrics fm
**                              int quad
**                              Point p
**                              int nodeWidth
**
**     Return Values:           Point
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private Point getTitleXY(String title, FontMetrics fm,
			                        int quad,Point p,int nodeWidth)
     {
	  Point tp = new Point();
	      
	  switch(quad)
	  {
	       case quadfivesix :
		                  tp.y = p.y + nodeWidth/2 + fm.getHeight();
				  tp.x = p.x - (fm.stringWidth(title)/2);
				  break;

	       case quadtwoone:
		                  tp.y = p.y - nodeWidth/2 - fm.getDescent();
				  tp.x = p.x - (fm.stringWidth(title)/2);
				  break;
				   
	       case quadsevenzero:
		                  tp.y = p.y + nodeWidth/2;
				  tp.x = p.x + nodeWidth;
				  break;
			
	       case quadfourthree:
		                  tp.y = p.y + nodeWidth/2;
				  tp.x = p.x - (fm.stringWidth(title) + 
						nodeWidth);
				  break;
			
	       case quadsix:
		                  tp.y = p.y + nodeWidth/2 + fm.getHeight();
				  tp.x = p.x - fm.stringWidth(title);
				  break;
	       case quadseven:
		                  tp.y = p.y + nodeWidth/2 + fm.getHeight();
				  tp.x = p.x;
				  break;

	       case quadthree:
		                  tp.y = p.y - nodeWidth/2 - fm.getDescent();
				  tp.x = p.x - fm.stringWidth(title);
				  break;

	       case quadtwo:
		                  tp.y = p.y - nodeWidth/2 - fm.getDescent();
				  tp.x = p.x;
				  break;

	       case quadeight:
		                  tp.y = p.y + fm.getAscent();
				  tp.x = p.x + nodeWidth;
				  break;			
	       case quadone:
		                  tp.y = p.y - nodeWidth/2;
				  tp.x = p.x + nodeWidth;
				  break;	
	       case quadfive:
		                  tp.y = p.y + fm.getAscent();
				  tp.x = p.x - (fm.stringWidth(title) + 
						nodeWidth);
				  break;
	       case quadfour:
		                  tp.y = p.y - fm.getDescent();
				  tp.x = p.x - (fm.stringWidth(title) + 
						nodeWidth);
				  break;
	  }
	       
	  // Handle cases when tp.x or tp.y become an invalid value;
	  return tp;
     }

/******************************************************************************
**
**     Method name:             getQuadP1 (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Point p1 - start point of the line
**                              Point p2 - end point of the line
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public static int getQuadP1(Point p1, Point p2)
     {
	  if (p1.x <= p2.x)
	  {
	       if (p1.y >= p2.y)
	       {
		    return quadone;
	       }
	       else
	       {
		    return quadfour;
	       }
	  }
	  else
	  {
	       if (p1.y > p2.y)
	       {
		    return quadtwo;
	       }
	       else
	       {
		    return quadthree;
	       }
	  }
     }

/******************************************************************************
**
**     Method name:             buildLineMatrix (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private void buildLineMatrix()
     {
	  LineMatrix lm;
	  String src;
	  String dest;
	  int numofLines = lines.size();
	  for (int i=0; i<numofLines; i++)
	  {
	       line = (Line)lines.elementAt(i);
	       src = line.getSrc();
	       dest = line.getDest();
	       lm = (LineMatrix)lineMatrix.get(src+dest);
	       if (lm == null)
	       {
		    lineMatrix.put(src+dest,new LineMatrix(src,dest));
	       }
	       else
	       {
		    lm.incrValue();
	       }
	  }
     }

/******************************************************************************
**
**     Method name:             getQuadrant (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Point p1 - start point of the line
**                              Point p2 - end point of the line
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private int getQuadrant(Point p1, Point p2)
     {
	  int quad = getQuadP1(p1,p2);
	  float height = new Float(Math.abs(p1.y - p2.y)).floatValue();
	  float width = new Float(Math.abs(p1.x - p2.x)).floatValue();
	  
	  if (width == 0)
	  {
	       if (p1.y < p2.y)
	       {
		    return quadsix;
	       }
	       else
	       {
		    return quadthree;
	       }
	  }
	  else if (height == 0)
	  {
	       if (p1.x < p2.x)
	       {
		    return quadone;
	       }
	       else
	       {
		    return quadfive;
	       }
	  }
	  else
	  {
	       switch(quad)
	       {
	            case quadone:
	            case quadthree:
			            if ( height > width )
				    {
					 return 2*quad+1;
				    }
				    else
				    {
					 return 2*quad;
				    }
	            case quadtwo:
	            case quadfour:
			            if ( width > height )
				    {
					 return 2*quad+1;
				    }
				    else
				    {
					 return 2*quad;
				    }
	       }
	  }
	  
	  return quadone;
     }

/******************************************************************************
**
**     Method name:             difflon (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              float lon1
**                              float lon2
**
**     Return Values:           float
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private float difflon(float lon1, float lon2)
     {
	  if ((lon1 > 0 && lon2 > 0) || (lon1 < 0 && lon2 < 0))
	  {
	       return Math.abs(lon1) - Math.abs(lon2);
	  }
	  else
	  {
	       return (360 - (Math.abs(lon1) + Math.abs(lon2)));
	  } 
     }


/******************************************************************************
**
**     Method name:             difflat (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              float lat1
**                              float lat2
**
**     Return Values:           float
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private float difflat(float lat1, float lat2)
     {
	  if ((lat1 > 0 && lat2 > 0) || (lat1 < 0 && lat2 < 0))           
	  {
	       return Math.abs(Math.abs(lat1) - Math.abs(lat2));
	  }
	  else
	  {
	       return Math.abs(lat1) + Math.abs(lat2);
	  }
     }
	
	  
/******************************************************************************
**
**     Method name:             pointWithinComponent (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Point p
**
**     Return Values:           boolean
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private boolean pointWithinComponent(Point p)
     {
	  if ( p.x < 0 || p.x > r.width) 
	  {
	       return false;
	  }
	  else if (p.y < 0 || p.y > r.height)	 
	  {
	       return false;
	  }
	  else
	  {
	       return true;
	  }
     }


/******************************************************************************
**
**     Method name:             comparelon (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              float lon1
**                              float lon2 
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private int comparelon(float lon1, float lon2)
     {
	  if ((lon1 < 0 && lon2 > 0) || (lon1 > 0 && lon2 < 0))
	  {
	       return new Float(Math.abs(lon1) + Math.abs(lon2)).intValue();
	  }
	       
	  return 0;
     }
	
/******************************************************************************
**
**     Method name:             stringToLat (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              String lat
**
**     Return Values:           Double
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public Double stringToLat(String lat)
     {
	  Double latd = new Double(new Double(lat).doubleValue());
	  return latd;
     }
	
/******************************************************************************
**
**     Method name:             stringToLon (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              String lon
**
**     Return Values:           Double
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public Double stringToLon(String lon)
     {
	  Double lond = new Double(new Double(lon).doubleValue());
	  return lond;
     }
	 
/******************************************************************************
**
**     Method name:             roundNum (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              double d
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String roundNum(double d)
     {
	  int i;
	  d = Math.abs(d);
	  i = (int)(d*100.00)%100;
	  return  (int)d + "." + i;
     }

/******************************************************************************
**
**     Method name:             pointToLatLon (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Point p
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String pointToLatLon(Point p)
     {
	  double lat = aLat*(p.y) + bLat;		
	  double lon = aLon*(p.x) + bLon;
	  
	  if (lon > 180)
	  {
	       lon = lon - 360.0;
	  }

	  return roundNum(lat) + ((lat < 0.0) ? "S" :"N") + ", " 
	         + roundNum(lon) + ((lon < 0.0) ? "W" :"E");
     }

    
/******************************************************************************
**
**     Method name:             latLonToPoint (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              float lat
**                              float lon
**
**     Return Values:           Point
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public Point latLonToPoint(float lat, float lon)
     {    
	  Point p = new Point();
	  p.y = new Float((lat-bLat)/aLat + 0.5).intValue();
	  p.x = new Float((lon-bLon)/aLon + 0.5).intValue();
	  return p;
     }

/******************************************************************************
**
**     Method name:             getAppletWidth (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public static int getAppletWidth()
     {
	  return r.width;
     }

/******************************************************************************
**
**     Method name:             mouseDragged (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              MouseEvent e
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void mouseDragged(MouseEvent e)
     {

     }
   
/******************************************************************************
**
**     Method name:             mouseMoved (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              MouseEvent e
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void mouseMoved(MouseEvent e)
     {
	  mouseP = e.getPoint();

	  matched_regions.removeAllElements();

	  Enumeration enode = nodes.elements();
	  while(enode.hasMoreElements())
	  {
	       node = (Node)enode.nextElement();
	      
	       if (node.withinBoundary(mouseP))
	       {
		    matched_regions.addElement(node);
	       }
	  }

	  int numofLines = lines.size();
	  for ( int i=0; i < numofLines; i++)
	  {
	       line = (Line)lines.elementAt(i);

	       if (line.withinBoundary(mouseP))
	       {
		    matched_regions.addElement(line);
	       }
	  }

	  if (matched_regions.size() > 0)
	  {
	       mouseOver = true;
	       repaint();
	  }
	  else if (bufferChanged)
	  {
	      mouseOver = false;
	      bufferChanged = false;
	      repaint();
	  }

	  showStatus("");
     }

/******************************************************************************
**
**     Method name:             drawNodLin (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Graphics g - the graphics context describing 
**                                           the graphics environment the 
**                                           applet is running in.
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void drawNodLin(Graphics g)
     {
	  Object msObj;
	  int dist;
	  NodLin tmpnodLin;
	  URL url;

	  matched_regions.trimToSize();
	  if (matched_regions.size() == 1)
	  {
	       dist = 
		    ((NodLin)matched_regions.elementAt(0)).getDistance(mouseP);

	       if (dist >= 0 && dist < 3)
	       {
		    tmpnodLin = ((NodLin)matched_regions.elementAt(0));

		    if (bufferChanged && tmpnodLin != nodLin)
		    {
			 g.drawImage(mapbuffer, 0,0, r.width, r.height,this);
			 tmpnodLin.drawSelf(g);
			 nodLin = tmpnodLin;
		    }
		    else
		    {
			 tmpnodLin.drawSelf(g);
			 nodLin = tmpnodLin;			 
		    }

		    url = tmpnodLin.getURL();

		    bufferChanged = true;

		    String sbval;
		    
		    sb.setNumber(0);

		    if ((sbval = tmpnodLin.toString()) != "")
		    {
			 sb.setString(sbval,0);
			 if (url != null)
			 {
			      showStatus(url.toString());
			      sb.setString(url.toString(),1);
			      sb.setString(tmpnodLin.getDesc(),2);
			 }			 
			 else
			 {
			      sb.setString(tmpnodLin.getDesc(),1);
			 }
			 
		    }
		    else
		    {
			 sb.setString(tmpnodLin.getDesc(),0);
		    }
		    sb.draw(g);
	       }
	       else if(bufferChanged)
	       {
		    g.drawImage(mapbuffer, 0,0, r.width, r.height,this);
		    bufferChanged = false;		   
	       }
	  }
	  else if(matched_regions.size() > 1)
	  {
	       int minI=0;
	       int tmpdist;

	       int numofRegions = matched_regions.size();
	       dist = 
		   ((NodLin)matched_regions.elementAt(0)).getDistance(mouseP);

	       for (int i=1; i < numofRegions; i++)
	       {
		 tmpdist = 
		    ((NodLin)matched_regions.elementAt(i)).getDistance(mouseP);

		    if ( tmpdist < dist)
		    {
			 minI = i;
			 dist = tmpdist;
		    }
	       }

	       if (dist >= 0 && dist < 3)
	       {
		    tmpnodLin = ((NodLin)matched_regions.elementAt(minI));

		    if (bufferChanged && tmpnodLin != nodLin)
		    {
			 g.drawImage(mapbuffer, 0,0, r.width, r.height,this);
			 tmpnodLin.drawSelf(g);
			 nodLin = tmpnodLin;
		    }
		    else
		    {
			 tmpnodLin.drawSelf(g);
			 nodLin = tmpnodLin;			 
		    }

		    url = tmpnodLin.getURL();

		    bufferChanged = true;

		    String sbval;
		    sb.setNumber(0);
		    if ((sbval = tmpnodLin.toString()) != "")
		    {
			 sb.setString(sbval,0);
			 if (url != null)
			 {
			      showStatus(url.toString());
			      sb.setString(url.toString(),1);
			      sb.setString(tmpnodLin.getDesc(),2);
			 }			 
			 else
			 {
			      sb.setString(tmpnodLin.getDesc(),1);
			 }
			 
		    }
		    else
		    {
			 sb.setString(tmpnodLin.getDesc(),0);
		    }
		    sb.draw(g);
	       }
	       else if(bufferChanged)
	       {
		    g.drawImage(mapbuffer, 0,0, r.width, r.height,this);
		    bufferChanged = false;		   
	       }
	  }

	  mouseOver = false;
     } 


/******************************************************************************
**
**     Method name:             mouseClicked (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              MouseEvent e - 
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void mouseClicked(MouseEvent e)
     {
	  NodLin tmpnodLin;
	  int dist;
	  mouseP = e.getPoint();

	  matched_regions.removeAllElements();

	  Enumeration enode = nodes.elements();
	  while(enode.hasMoreElements())
	  {
	       node = (Node)enode.nextElement();
	      
	       if (node.withinBoundary(mouseP))
	       {
		    matched_regions.addElement(node);
	       }
	  }

	  int numofLines = lines.size();
	  for ( int i=0; i < numofLines; i++)
	  {
	       line = (Line)lines.elementAt(i);

	       if (line.withinBoundary(mouseP))
	       {
		    matched_regions.addElement(line);
	       }
	  }

	  if (matched_regions.size() == 1)
	  {
	       dist = 
		   ((NodLin)matched_regions.elementAt(0)).getDistance(mouseP);

	       if (dist >= 0 && dist < 3)
	       {
		    tmpnodLin = ((NodLin)matched_regions.elementAt(0));
		    URL url = tmpnodLin.getURL();

		    if (url != null)
		    {
			 AppletContext ac = getAppletContext();
			 ac.showDocument(url,"_blank");
		    }
	       }
	  }
	  else if(matched_regions.size() > 1)
	  {
	       int minI=0;
	       int tmpdist;

	       int numofRegions = matched_regions.size();
	       dist = 
		   ((NodLin)matched_regions.elementAt(0)).getDistance(mouseP);

	       for (int i=1; i < numofRegions; i++)
	       {
		 tmpdist = 
		    ((NodLin)matched_regions.elementAt(i)).getDistance(mouseP);

		    if ( tmpdist < dist)
		    {
			 minI = i;
			 dist = tmpdist;
		    }
	       }

	       if (dist >= 0 && dist < 3)
	       {
		    tmpnodLin = ((NodLin)matched_regions.elementAt(minI));
		    URL url = tmpnodLin.getURL();

		    if (url != null)
		    {
			 AppletContext ac = getAppletContext();
			 ac.showDocument(url,"_blank");
		    }
	       }
	  }
     }

     public void mousePressed(MouseEvent e)
     {
 
     }
 
     public void mouseReleased(MouseEvent e)
     {

     }

     public void mouseEntered(MouseEvent e)
     {

     } 

     public void mouseExited(MouseEvent e)
     {

     }


     class LineMatrix
     {
	  int numofLines;

	  public LineMatrix(String src, String dest)
	  {
	       numofLines = 1;
	  }

	  public void incrValue()
	  {
	       numofLines++;
	  }

	  public void decrValue()
	  {
	       numofLines--;
	  }

	  public int getValue()
	  {
	       return numofLines;
	  }
     }
}
