/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GeoPlot tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GeoPlot tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant NCR-9711092, and by CAIDA members.

Report bugs and suggestions to ram@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    Key.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, May 30th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/

import java.util.StringTokenizer;
import java.util.NoSuchElementException;

public class Key
{
     float lowerbound;
     float upperbound;
     String value;
     String label;

/******************************************************************************
**
**     Constructor
**
**     Purpose:                 
**    
**     Parameters:              String data
**
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public Key(String data)
     {
          try 
	  {
	       StringTokenizer tokenizer = new StringTokenizer(data,":");

	       int numofTokens = tokenizer.countTokens();

	       String token = tokenizer.nextToken();
	       if ( token.indexOf("&&") > 0 )
	       {
		    /* Range specified */
		    StringTokenizer bounds = new StringTokenizer(token,"&&");

		    try
		    {
		         String temp = bounds.nextToken();
			 lowerbound = Float.valueOf(temp.trim()).floatValue();
		    }
		    catch (NumberFormatException nfe)
		    {
		         System.err.println("NumberFormatException LB");
			 lowerbound = Float.POSITIVE_INFINITY;
		    }
		   
		    try
		    {
		         String temp = bounds.nextToken();
			 upperbound = Float.valueOf(temp.trim()).floatValue();
		    }
		    catch (NumberFormatException nfe)
		    {
		         System.err.println("NumberFormatException UB");
			 upperbound = Float.POSITIVE_INFINITY;
		    }	      
	       }
	       else
	       {
		    /* Single value specified */ 
		    try
		    {
		         lowerbound = Float.valueOf(token).floatValue();
		    }
		    catch (NumberFormatException nfe)
		    {
		         lowerbound = Float.POSITIVE_INFINITY;
		    }
	      
		    upperbound = lowerbound;
	       }

	       value = tokenizer.nextToken().trim();

	       if (numofTokens > 2)
	       {
		    label = tokenizer.nextToken().trim();
	       }
	       else
	       {
		    label = "";
	       }
	  }
	  catch (NoSuchElementException nsee)
	  {
	       lowerbound = upperbound = Float.POSITIVE_INFINITY;
	       value = null;
	       label = "";
	  }
     }
    

/******************************************************************************
**
**     Method name:             withinbounds (class Key)
**
**     Purpose:                 
**    
**     Parameters:              float val
**
**     Return Values:           boolean
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/
 
     public boolean withinbounds(float val)
     {
	  if (val >= lowerbound && val <= upperbound)
	  {
	       return true;
	  }
	  else
	  {
	       return false;
	  }
     }

/******************************************************************************
**
**     Method name:             toString (class Key)
**
**     Purpose:                 
**    
**     Parameters:              None
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String toString()
     {
	  if (label == "")
	  {
	       return (lowerbound + " -- " + upperbound);
	  } 
	  else
	  {
	       return label;
	  }
     }

/******************************************************************************
**
**     Method name:             getValue (class Key)
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           String 
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String getValue()
     {
	  return value;
     }
}
