/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GeoPlot tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GeoPlot tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant NCR-9711092, and by CAIDA members.

Report bugs and suggestions to ram@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    Line.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, May 19th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/

import java.awt.Point;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Graphics;

public class Line extends NodLin
{
     String src;
     String dest;
     int direc;
     int type=0;

     Rectangle r[];
     Point p[];
     float m[];
     float c[];

/******************************************************************************
**
**     Constructor
**
**     Purpose:                 
**    
**     Parameters:              String src 
**                              String dest 
**                              int size 
**                              String color
**
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public Line(String src, String dest, int size, String color)
     {
	  super(size, color);
	  this.src = src;
	  this.dest = dest;
     }

/******************************************************************************
**
**     Constructor
**
**     Purpose:                 
**    
**     Parameters:              String src 
**                              String dest 
**                              int size 
**                              String color
**                              String data
**
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public Line(String src,String dest, int size, String color, String data)
     {
	  super(size, color, data);
	  
	  this.src = src;
	  this.dest = dest;

	  int start;
	  if ((start = data.indexOf("R:")) > 0 || data.indexOf("r:") > 0)
	  {
	       if (start <= 0)
	       {
		    start = data.indexOf("r:");
	       }
		  
	       try
	       {
		    this.direc = new Integer(parseData(start,
					      data.length(),data)).intValue();
	       }
	       catch(NumberFormatException e)
	       {
		    this.direc = 0;
	       }
	  }
	  else
	  {
	       this.direc = 0;
	  }
     }

/******************************************************************************
**
**     Method name:             setType (class Line)
**
**     Purpose:                 
**    
**     Parameters:              int type
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void setType(int type)
     {
	  this.type = type;
     }

    
/******************************************************************************
**
**     Method name:             setPoint (class Line)
**
**     Purpose:                 
**    
**     Parameters:              Point p1, Point p2
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void setPoint(Point p1, Point p2)
     {
	  this.p = new Point[2];
	  this.p[0] = p1;
	  this.p[1] = p2;
	  
	  r = new Rectangle[1];
	  m = new float[1];
	  c = new float[1];

	  if ( (p[0].x == p[1].x) || (p[0].y ==  p[1].y) )
	  {
	       r[0] = splCaseHorVerLines(p[0],p[1]);
	       m[0] = 0;
	       c[0] = 0;
	  }
	  else
	  {
	       m[0] = new Float(p1.y - p2.y).floatValue()/
		                       new Float(p1.x - p2.x).floatValue();
	       c[0] = new Integer(p1.y).floatValue() - 
		           (new Float(m[0]).floatValue() * 
			             new Integer(p1.x).floatValue());
	  
	       int X = getMin(p,0);
	       int Y = getMin(p,1);
	  
	       r[0] = new Rectangle(X,Y,getMax(p,0) - X,getMax(p,1) - Y);
	  }
     }
    
/******************************************************************************
**
**     Method name:             setPoint (class Line)
**
**     Purpose:                 
**    
**     Parameters:              Point p[], int typ
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void setPoint(Point p[], int typ)
     {
	  this.p = p;
	 
	  if (typ == 0)
	  {
	       r = new Rectangle[1];
	       Rectangle rtmp = new Rectangle();
	       
	       int X = getMin(p,0);
	       int Y = getMin(p,1);
	       r[0] = new Rectangle(X,Y,getMax(p,0) - X,getMax(p,1) - Y);
	       
	       m = new float[p.length-1];
	       c = new float[p.length-1];

	       for (int i=0; i<p.length-1; i++)
	       {
		    if ( (p[i].y == p[i+1].y)  || (p[i].x ==  p[i+1].x) )
		    {
			 rtmp = splCaseHorVerLines(p[i],p[i+1]);
			 r[0] = r[0].union(rtmp);
			 m[i] = 0;
			 c[i] = 0;
		    }
		    else
		    {
			 m[i] = new Float(p[i].y - p[i+1].y).floatValue()/
			        new Float(p[i].x - p[i+1].x).floatValue();
			 c[i] = new Integer(p[i].y).floatValue() - 
			        (new Float(m[i]).floatValue() * 
				 new Integer(p[i].x).floatValue());
		    }
	       }	      
	  }
	  else if (typ == 8)
	  {
	       r = new Rectangle[1];
	       r[0] = new Rectangle(p[0].x-p[1].x,
				    p[0].y-p[1].x,2*p[1].x,2*p[1].x);
	  }
	  else
	  {
	       r = new Rectangle[2];
	       Rectangle rtmp = new Rectangle();
	       int X;
	       int Y;

	       if (p.length == 4)
	       {
		    Point ptemp[] = new Point[2];
		    ptemp[0] = p[0];
		    ptemp[1] = p[1];
		    
		    m = new float[2];
		    c = new float[2];

		    X = getMin(ptemp,0);
		    Y = getMin(ptemp,1);
		    r[0] = new Rectangle(X,Y,getMax(ptemp,0) - X,
					 getMax(ptemp,1) - Y);

		    if ( (p[0].y == p[1].y) || (p[0].x ==  p[1].x))
		    {
			 rtmp = splCaseHorVerLines(p[0],p[1]);
			 r[0] = r[0].union(rtmp);
			 m[0] = 0;
			 c[0] = 0;
		    }
		    else
		    {
			 m[0] = new Float(p[0].y - p[1].y).floatValue()/
			        new Float(p[0].x - p[1].x).floatValue();
			 c[0] = new Integer(p[0].y).floatValue() - 
			        (new Float(m[0]).floatValue() * 
				 new Integer(p[0].x).floatValue());
		    }
		    

		    ptemp[0] = p[2];
		    ptemp[1] = p[3];
		    
		    X = getMin(ptemp,0);
		    Y = getMin(ptemp,1);
		    r[1] = new Rectangle(X,Y,getMax(ptemp,0) - X,
					 getMax(ptemp,1) - Y);		   
		    
		    if ( (p[2].x == p[3].x) || (p[2].y ==  p[3].y) )
		    {
			 rtmp = splCaseHorVerLines(p[2],p[3]);
			 r[1] = r[1].union(rtmp);
			 m[1] = 0;
			 c[1] = 0;
		    }
		    else
		    {
			 m[1] = new Float(p[2].y - p[3].y).floatValue()/
			        new Float(p[2].x - p[3].x).floatValue();
			 c[1] = new Integer(p[2].y).floatValue() - 
			        (new Float(m[1]).floatValue() * 
				 new Integer(p[2].x).floatValue());
		    }
	       }
	       else if(p.length == 6)
	       {
		    //xxx --- investigate pure horizontal || vertical
		   
		    Point ptemp[] = new Point[3];
		    m = new float[p.length-1];
		    c = new float[p.length-1];

		    ptemp[0] = p[0];
		    ptemp[1] = p[1];
		    ptemp[2] = p[2];
		    
		    X = getMin(ptemp,0);
		    Y = getMin(ptemp,1);
		    r[0] = new Rectangle(X,Y,getMax(ptemp,0) - X,
					 getMax(ptemp,1) - Y);

		    ptemp[0] = p[3];
		    ptemp[1] = p[4];
		    ptemp[2] = p[5];
		    
		    X = getMin(ptemp,0);
		    Y = getMin(ptemp,1);
		    r[1] = new Rectangle(X,Y,getMax(ptemp,0) - X,
					 getMax(ptemp,1) - Y);	

		    for (int i=0; i<p.length-1; i++)
		    {
			 if ( i != 2 && (p[i].y == p[i+1].y)  
			      || (p[i].x ==  p[i+1].x) )
			 {
			      rtmp = splCaseHorVerLines(p[i],p[i+1]);
			      if (i > 2)
			      {
				   r[1] = r[1].union(rtmp);
			      }
			      else
			      {
				   r[0] = r[0].union(rtmp);
			      }
			      m[i] = 0;
			      c[i] = 0;
			 }
			 else
			 {
			      m[i] = new Float(p[i].y - p[i+1].y).floatValue()/
				     new Float(p[i].x - p[i+1].x).floatValue();
			      c[i] = new Integer(p[i].y).floatValue() - 
				     (new Float(m[i]).floatValue() * 
				      new Integer(p[i].x).floatValue());
			 }
		    }
	       }
	  }
     }

/******************************************************************************
**
**     Method name:             splCaseHorVerLines (class Line)
**
**     Purpose:                 
**    
**     Parameters:              Point p1, Point p2
**
**     Return Values:           Rectangle
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, July 13th, 1999
**
******************************************************************************/

     private Rectangle splCaseHorVerLines(Point p1, Point p2)
     {
	  int X;
	  int Y;

	  if (p1.x == p2.x )
	  { 
	       Y = Math.min(p1.y,p2.y);
	       
	       X = p1.x - 3 - size/2;

	       if ( X < 0)
	       {
		    X = 0;
	       }

	       int Xbot; 
	       if ( (p1.x + 3 + size/2) > GeoPlot.r.width)
	       {
		    Xbot = GeoPlot.r.width - p1.x;
	       }
	       else
	       {
		    Xbot = 3 + size/2;
	       }
	       
	       return new Rectangle(X,Y, (p1.x - X) + Xbot, 
				    Math.abs(p1.y - p2.y)); 
	  }
	  else if(p1.y ==  p2.y)
	  {
	       X = Math.min(p1.x,p2.x);
	       
	       Y = p1.y - 3 - size/2;

	       if ( Y < 0)
	       {
		    Y = 0;
	       }

	       int Ybot; 
	       if ( (p1.y + 3 + size/2) > GeoPlot.r.height)
	       {
		    Ybot = GeoPlot.r.height - p1.y;
	       }
	       else
	       {
		    Ybot = 3 + size/2;
	       }
	       
	       return new Rectangle(X,Y, Math.abs(p1.x - p2.x), 
				    (p1.y - Y) + Ybot);   
	  }

	  return new Rectangle();
     }

/******************************************************************************
**
**     Method name:             drawSelf (class Line)
**
**     Purpose:                 
**    
**     Parameters:              Graphics g
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void drawSelf(Graphics g)
     {
          g.setColor(GeoPlot.fgcolor);
 
	  if (type == 8)
	  {
	       GeoPlot.drawSCircle(g,size,p);
	  }
	  else
	  {
	       GeoPlot.drawSLine(g, size, p, direc);
	  }
     }
    
/******************************************************************************
**
**     Method name:             getDistance (class Line)
**
**     Purpose:                 
**    
**     Parameters:              Point pt
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public int getDistance(Point pt)
     {
	  int x2;
	  int y2;
	  int X;
	  int Y;
	  int minDist=GeoPlot.r.height;
	  int tmpDist=GeoPlot.r.height;
	  Point ptmp[] = new Point[2];
	  boolean first = true;

	  if (type == 8)
	  {
	       if (r[0].contains(pt))
	       {
		    int a = Math.abs(pt.x - (r[0].x + p[1].x));
		    int b = Math.abs(pt.y - (r[0].y + p[1].x));

		    int c = new Long(Math.round(Math.sqrt(
			    new Double(a*a +  b*b).doubleValue()))).intValue();
		   
		    if (size == 1)
		    {
		         return Math.abs(p[1].x - c);
		    }
		    else
		    {
		 	if ( c < p[1].x && c > p[1].x - size)
			{
			     return 0;
			}

			return Math.min(Math.abs(c-p[1].x),
					Math.abs(c - p[1].x) - size);
		    }
	       }

	       return 3;
	  }
	  
	  for (int i=0; i<m.length; i++)
	  {
	       if (p[i].x == GeoPlot.r.width && p[i+1].x == 0 && m.length > 2)
	       {
		    continue;
	       }
	       else if(m.length == 2 && i == 1)
	       {
		    ptmp[0] = p[i+1];
		    ptmp[1] = p[i+2];
	       }
	       else
	       {
		    ptmp[0] = p[i];
		    ptmp[1] = p[i+1];
	       }

	       if (m[i] == 0 && c[i] == 0)
	       {
		    if ( p[i].x == p[i+1].x ) 
		    {
			 tmpDist = Math.abs(pt.x - p[i].x);
		    }
		    else if( p[i].y ==  p[i+1].y )
		    {
			 tmpDist = Math.abs(pt.y - p[i].y);
		    }
	       }
	       else if (pt.x < getMin(ptmp,0) || pt.x > getMax(ptmp,0))
	       {
		    continue;
	       }
	       else if(pt.y < getMin(ptmp,1) || pt.y > getMax(ptmp,1))
	       {
		    continue;
	       }
		    
	       if (!(m[i] == 0 && c[i] == 0))
	       {
		    x2 = Math.round((pt.y - c[i])/m[i]);
		    y2 = Math.round(pt.x * m[i] + c[i]);
		    X = Math.abs(pt.x - x2);
		    Y = Math.abs(pt.y - y2);
		    tmpDist = new Double(X * Y/ 
			       Math.sqrt(new Double(X*X + 
				       Y*Y).doubleValue())).intValue();	       
	       }

	       //System.err.println("tmpDist = " +tmpDist + "   i = " + i);
	       //System.err.println("----");

	       if (first)
	       {
		    if (tmpDist >= 0)
		    {
			 minDist = tmpDist;
		    }
		    first = false;
	       }
	       else if (tmpDist >= 0 && tmpDist < minDist)
	       {
		    minDist = tmpDist;
	       }
	  }
	  
	  // xxx - minDist calculation for the ends of lines type 4 and 6

	  if (size > 1 && minDist > 2)
	  {
	       if (minDist > size/2)
	       {
		   //System.err.println("***minDist = " + (minDist - size/2));
		    return (minDist - size/2);
	       }
	       //System.err.println("size = " + size);
	       //System.err.println("---minDist = " + minDist);
	       //System.err.println("minDist 1 returned");
	       return 1;
	  }

	  //System.err.println("minDist = " + minDist);
	  //System.err.println("*******************************************");

	  return minDist;
     }



/******************************************************************************
**
**     Method name:             withinBoundary (class Line)
**
**     Purpose:                 
**    
**     Parameters:              Point pt
**
**     Return Values:           boolean
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public boolean withinBoundary(Point pt)
     {   
	 if (r != null)
	 {
	      for (int i=0; i<r.length; i++)
	      {	 
		   if ( r[i] != null && r[i].contains(pt) )
		   {
		        return true;
		   }
	      }
	 }
	 
	 return false;
     }

/******************************************************************************
**
**     Method name:             getMin (class )
**
**     Purpose:                 This function returns the maximum x/y value 
**                              from the given set of points.
**    
**     Parameters:              Point pt[] - The given set of points
**                              int XY - determines which to maximum to return.
**                                       0 = x,  1 = y.
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private int getMin(Point pt[], int XY)
     {
	  int min;
	 
	  if (XY == 0)
	  {
	       min = pt[0].x;
	 
	       for (int i=1; i<pt.length; i++)
	       {
		    min = Math.min(min,pt[i].x);
	       }
	  }
	  else
	  {
	       min = pt[0].y;
	       
	       for (int i=1; i<pt.length; i++)
	       {
		    min = Math.min(min,pt[i].y);
	       }
	  }

	  return min;
     }

/******************************************************************************
**
**     Method name:             getMax (class Line)
**
**     Purpose:                 This function returns the maximum x/y value 
**                              from the given set of points.
**     
**     Parameters:              Point pt[] - The given set of points
**                              int XY - determines which to maximum to return.
**                                       0 = x,  1 = y.
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     private int getMax(Point pt[], int XY)
     {
	  int max;
	 
	  if (XY == 0)
	  {
	       max = pt[0].x;
	 
	       for (int i=1; i<pt.length; i++)
	       {
		    max = Math.max(max,pt[i].x);
	       }
	  }
	  else
	  {
	       max = pt[0].y;
	       
	       for (int i=1; i<pt.length; i++)
	       {
		    max = Math.max(max,pt[i].y);
	       }
	  }

	  return max;
     }

/******************************************************************************
**
**     Method name:             getType (class Line)
**
**     Purpose:                 Returns the type of the line, valid type values
**                              are 1,3,5.
**    
**     Parameters:              NONE
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public int getType()
     {
	  return type;
     }

/******************************************************************************
**
**     Method name:             getSrc (class Line)
**
**     Purpose:                 Returns the source node of this line.
**    
**     Parameters:              NONE
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String getSrc()
     {
	  return src;
     }

/******************************************************************************
**
**     Method name:             getDest (class Line)
**
**     Purpose:                 Returns the destionation node of this line.
**    
**     Parameters:              NONE
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String getDest()
     {
	  return dest;
     }

/******************************************************************************
**
**     Method name:             getDirec (class Line)
**
**     Purpose:                 Returns the direction in which the arrow should
**                              be drawn.
**    
**     Parameters:              NONE
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public int getDirec()
     {
	 return direc;
     }

/******************************************************************************
**
**     Method name:             toStringPoint (class Line)
**
**     Purpose:                 Returns of the string eqivalent of the points 
**                              that make up this line.
**    
**     Parameters:              NONE
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String toStringPoint()
     {
	  String temp="";
	 
	  for (int i=0; i<p.length; i++)
	  {
	       temp += "p["+i+"] = " + p[i].toString();
	  }

	  return temp;
     }

/******************************************************************************
**
**     Method name:             toString (class Line)
**
**     Purpose:                 Returns the string representation of the Color
**                              value and Size value of the give line.
**    
**     Parameters:              NONE
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String toString()
     {
	  String tmp="";
	 
	  if (Cvalue != Float.POSITIVE_INFINITY)
	  {
	       tmp = "Color = " + Cvalue + " ";
	  }

	  if (Svalue != Float.POSITIVE_INFINITY)
	  {
	       tmp += "Size = " + Svalue;
	  }

          return tmp;
     }

/******************************************************************************
**
**     Method name:             setDirec  (class Line)
**
**     Purpose:                 
**    
**     Parameters:              int mult
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public int setDirec(int mult)
     {
	  if ( direc > 0)
	  {
	       direc = direc + (3*mult);
	  }
	  return direc;
     }

/******************************************************************************
**
**     Method name:             chngDirec  (class Line)
**
**     Purpose:                 
**    
**     Parameters:              int direc
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void chngDirec(int direc)
     {     
	  this.direc = direc;
     }
}
