/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GeoPlot tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GeoPlot tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant NCR-9711092, and by CAIDA members.

Report bugs and suggestions to ram@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    NodLin.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, May 19th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/

import java.awt.Color;
import java.util.Hashtable;
import java.awt.Graphics;
import java.awt.Point;
import java.util.StringTokenizer;
import java.net.URL;
import java.net.MalformedURLException;

public class NodLin
{
     String title;
     int size;
     float Cvalue;
     float Svalue;
     Color color;
     URL url;    // change to type URL
     String descp;

     static Hashtable colormap = new Hashtable();

     static
     {    
	  colormap.put("black",Color.black);
	  colormap.put("blue",Color.blue);
	  colormap.put("cyan",Color.cyan); 
	  colormap.put("darkGray",Color.darkGray); 
	  colormap.put("gray",Color.gray); 
	  colormap.put("green",Color.green); 
	  colormap.put("lightGray",Color.lightGray); 
	  colormap.put("magenta",Color.magenta); 
	  colormap.put("orange",Color.orange); 
	  colormap.put("pink",Color.pink); 
	  colormap.put("red",Color.red); 
	  colormap.put("white",Color.white); 
	  colormap.put("yellow",Color.yellow); 
     }

/******************************************************************************
**
**     Constructor
**
**     Purpose:                 
**    
**     Parameters:              int size
**                              String color
**
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public NodLin(int size, String color)
     {
	  title = "";
	  this.size = size;
	  Cvalue = Float.POSITIVE_INFINITY;
	  Svalue = Float.POSITIVE_INFINITY;
	  this.color = getColor(color);
	  url = null;
	  descp = "";
     }

/******************************************************************************
**
**     Constructor
**
**     Purpose:                 
**    
**     Parameters:              int size 
**                              String color
**                              String data
**
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public NodLin(int size, String color, String data)
     {
	  int length = data.length();
	  int start;
	  String str;

	  if ((start = data.indexOf("T:")) > 0 || data.indexOf("t:") > 0)
	  {
	       if (start <= 0)
	       {
		    start = data.indexOf("t:");
	       }
	       
	       str = parseData(start,length,data);
	       if (str != null)
	       {
		    this.title = str;
	       }
	       else
	       {
		    this.title = "";
	       }
	  }
	  else
	  {
	       this.title = "";
	  }

	  if ((start = data.indexOf("SZ:")) > 0 || data.indexOf("sz:") > 0)
	  {
	       if (start <= 0)
	       {
		    start = data.indexOf("sz:");
	       }
		 
	       try
	       {
		    this.size = new Integer(parseData(start,
						      length,data)).intValue();
	       }
	       catch(NumberFormatException e)
	       {
		    this.size = size;
	       }
	  }
	  else
	  {
	       this.size = size;
	  }

	  if ((start = data.indexOf("VC:")) > 0 || data.indexOf("vc:") > 0)
	  {
	       if (start <= 0)
	       {
		    start = data.indexOf("vc:");
	       }
		 
	       str = parseData(start,length,data);
	       if (str != null)
	       {
		    try
		    {
			 this.Cvalue = Float.valueOf(str).floatValue();
		    }
		    catch(NumberFormatException nfe)
		    {
			 this.Cvalue = Float.POSITIVE_INFINITY;
		    }
	       }
	       else
	       {
		    this.Cvalue = Float.POSITIVE_INFINITY;
	       }
	  }
	  else
	  {
	       this.Cvalue = Float.POSITIVE_INFINITY;
	  }

	  if ((start = data.indexOf("VS:")) > 0 || data.indexOf("vs:") > 0)
	  {
	       if (start <= 0)
	       {
		    start = data.indexOf("vs:");
	       }
		 
	       str = parseData(start,length,data);
	       if (str != null)
	       {
		    try
		    {
			 this.Svalue = Float.valueOf(str).floatValue();
		    }
		    catch(NumberFormatException nfe)
		    {
			 this.Svalue = Float.POSITIVE_INFINITY;
		    }
	       }
	       else
	       {
		    this.Svalue = Float.POSITIVE_INFINITY;
	       }
	  }
	  else
	  {
	       this.Svalue = Float.POSITIVE_INFINITY;
	  }

	  if ((start = data.indexOf("CL:")) > 0 || data.indexOf("cl:") > 0)
	  {
	       if (start <= 0)
	       {
		    start = data.indexOf("cl:");
	       }
		 
	       str = parseData(start,length,data);
	       if (str != null)
	       {
		    this.color = getColor(str);
	       }
	       else
	       {
		    this.color = getColor(color);
	       }
	  }
	  else
	  {
	       this.color = getColor(color);
	  }

	  if ((start = data.indexOf("U:")) > 0 || data.indexOf("u:") > 0)
	  {
	       if (start <= 0)
	       {
		    start = data.indexOf("u:");
	       }
		 
	       str = parseData(start,length,data);
	       if (str != null)
	       {
		    try
		    {
			 this.url = new URL(str);
		    }
		    catch(MalformedURLException murle)
		    {
			 this.url = null;
		    }
	       }
	       else
	       {
		    this.url = null;
	       }
	  }
	  else
	  {
	       this.url = null;
	  }

	  if ((start = data.indexOf("D:")) > 0 || data.indexOf("d:") > 0)
	  {
	       if (start <= 0)
	       {
		    start = data.indexOf("d:");
	       }
		 
	       str = parseData(start,length,data);
	       if (str != null)
	       {
		    this.descp = str;
	       }
	       else
	       {
		    this.descp = "";
	       }
	  }
	  else
	  {
	       this.descp = "";
	  }
     }
    
/******************************************************************************
**
**     Method name:             parseData  (class NodLin)
**
**     Purpose:                 The function extracts the data portion from 
**                              the input string from a given starting point. 
**    
**     Parameters:              int start - index to the beginning of the 
**                                          parameter
**                              int length - length of the data string
**                              String data - the entire data itself
**
**     Return Values:           String - the data portion of the parameter
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String parseData(int start, int length, String data)
     {
	  char ch;
	  StringBuffer value = new StringBuffer();
	 
	  if ( data.charAt(start+1) == ':')
	  {
	       start += 2;
	  }
	  else if ( data.charAt(start+2) == ':')
	  {
	       start += 3;
	  }
	  else
	  {
	       // handle case here for bad input data
	  }
	 
	  for (int i=start; i < length; i++)
	  {
	       ch = data.charAt(i);
	       if (ch == ':' && !value.toString().toLowerCase().equals("http"))
	       {
		    value.setCharAt(value.length()-1,' ');
		    char tmp = value.charAt(value.length()-2);
		    if ( tmp == 'v' || tmp == 'V' || 
	                 tmp == 's' || tmp == 'S' ||
	                 tmp == 'c' || tmp == 'C' )
		    {    
		         value.setCharAt(value.length()-2,' '); 
		    }
		    break;
	       }
	       value.append(data.charAt(i));
	  }
	 
	  return (value.toString().trim());
     }

/******************************************************************************
**
**     Method name:             getColor (class NodLin)
**
**     Purpose:                 Converts a string representation of a color 
**                              to a Color object.
**    
**     Parameters:              String color - the string representation
**
**     Return Values:           Color - the Color object equivalent of color
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public static Color getColor(String color)
     {
	  StringTokenizer tokenizer;

	  // check to see if the color is represented as rgb values
	  if (!color.toLowerCase().startsWith("rgb"))
	  {
	       return (Color)colormap.get(color);
	  }
	  else
	  {
	       tokenizer = new StringTokenizer(color,"-");
	       if (tokenizer.countTokens() < 4)
	       {
		    return null; 
	       }
	       
	       tokenizer.nextToken();
	       try
	       {
		    return (new Color(Integer.valueOf(
					   tokenizer.nextToken()).intValue(),
				      Integer.valueOf(
					   tokenizer.nextToken()).intValue(),
				      Integer.valueOf(
					   tokenizer.nextToken()).intValue()));
	       }
	       catch(NumberFormatException e)
	       {
		    System.err.println
			("Handle numberformatexception nodlin getColor");
		    return null;
	       }
	  }
     }

/******************************************************************************
**
**     Method name:             isTitle (class NodLin)
**
**     Purpose:                 Determines whether a title exits
**    
**     Parameters:              NONE
**
**     Return Values:           boolean - true if title is non-empty and 
**                                        false otherwise.
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public boolean isTitle()
     {
	  if (title != "")
	  {
	       return true;
	  }
	  
	  return false;
     }
    
/******************************************************************************
**
**     Method name:             drawSelf (class NodLin)
**
**     Purpose:                 
**    
**     Parameters:              Graphics g
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void drawSelf(Graphics g)
     {

     }

/******************************************************************************
**
**     Method name:             getDistance (class NodLin)
**
**     Purpose:                 
**    
**     Parameters:              Point pt
**
**     Return Values:           int
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public int getDistance(Point pt)
     {
	  return 0;
     }

/******************************************************************************
**
**     Method name:             getTitle (class NodLin)
**
**     Purpose:                 Function returns the title of the current 
**                              object
**    
**     Parameters:              NONE
**
**     Return Values:           String - the title
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String getTitle()
     {
	  return title;
     }

/******************************************************************************
**
**     Method name:             getSize (class NodLin)
**
**     Purpose:                 Fuction returns the size of the current object
**    
**     Parameters:              NONE
**
**     Return Values:           int - the size of the object, thickness for 
**                                    lines and diameter for nodes
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public int getSize()
     {
	  return size;
     }

/******************************************************************************
**
**     Method name:             getCValue (class NodLin)
**
**     Purpose:                 Function returns the color value of the object
**    
**     Parameters:              NONE
**
 **     Return Values:           float - the color value
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public float getCValue()
     {
	  return Cvalue;
     }

/******************************************************************************
**
**     Method name:             getSValue (class NodLin)
**
**     Purpose:                 Function returns the size value of the object
**    
**     Parameters:              NONE
**
**     Return Values:           float - the size value
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
*****************************************************************************/

     public float getSValue()
     {
	  return Svalue;
     }

/******************************************************************************
**
**     Method name:             getColor (class NodLin)
**
**     Purpose:                 Function returns the color of the object
**    
**     Parameters:              NONE
**
**     Return Values:           Color - the color value
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public Color getColor()
     {
	  return color;
     }

/******************************************************************************
**
**     Method name:             getURL (class NodLin)
**
**     Purpose:                 Function returns the URL specified for the 
**                              object
**    
**     Parameters:              NONE
**
**     Return Values:           URL - the URL value
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public URL getURL()
     {
	  return url;
     }

/******************************************************************************
**
**     Method name:             getDesc (class NodLin)
**
**     Purpose:                 Function returns the description associated 
**                              with the current object.
**    
**     Parameters:              NONE
**
**     Return Values:           String - the description
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String getDesc()
     {
	  return descp;
     }

/******************************************************************************
**
**     Method name:             toString (class NodLin)
**
**     Purpose:                 Function represents the string representation  
**                              of the current object.
**    
**     Parameters:              NONE
**
**     Return Values:           String - the string equivalent of this object
**                                           
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String toString()
     {
	  return "";
     }


/******************************************************************************
**
**     Method name:             setSize  (class NodLin)
**
**     Purpose:                 
**    
**     Parameters:              int size
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void setSize(int size)
     {
	  this.size = size;
     }
}
