/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GeoPlot tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GeoPlot tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant NCR-9711092, and by CAIDA members.

Report bugs and suggestions to ram@caida.org.
**************************************************************************/


/******************************************************************************
**
**     File:                    Node.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, May 19th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/

import java.awt.Point;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Color;

public class Node extends NodLin
{
     int nodNum;
     float lat;
     float lon;
     int[] quad_matrix = new int[8];

     Rectangle r = null;

     static int labelorder[] = {5,6,2,1,7,0,4,3};

/******************************************************************************
**
**     Constructor
**
**     Purpose:                 
**    
**     Parameters:              int nodNum 
**                              String lat 
**                              String lon
**                              int size
**                              String color
**
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/
    
     public Node(int nodNum, String lat, String lon, int size, String color)
     {
	  super(size, color);
	
	  this.nodNum = nodNum;

	  try
	  {
	       this.lat = Float.valueOf(lat).floatValue();
	  }
	  catch(NumberFormatException e)
	  {
	       System.err.println("Handle numberformatexception node_lat");
	  }

	  try
	  {
	       this.lon = Float.valueOf(lon).floatValue();
	  }
	  catch(NumberFormatException e)
	  {
	       System.err.println("Handle numberformatexception node_lon");
	  }

	  for (int i=0; i<8; i++)
	  {
	       quad_matrix[i] = 0;
	  }
     }

/******************************************************************************
**
**     Constructor
**
**     Purpose:                 
**    
**     Parameters:              int nodNum
**                              String lat 
**                              String lon
**                              int size
**                              String color
**                              String data
**
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/
    

     public Node(int nodNum, String lat, String lon, int size, 
		                                     String color, String data)
     {
	  super(size, color, data);

	  this.nodNum = nodNum;

	  try
	  {
	       this.lat = Float.valueOf(lat).floatValue();
	  }
	  catch(NumberFormatException e)
	  {
	       System.err.println("Handle numberformatexception node_lat");
	  }

	  try
	  {
	       this.lon = Float.valueOf(lon).floatValue();
	  }
	  catch(NumberFormatException e)
	  {
	       System.err.println("Handle numberformatexception node_lon");
	  }

	  for (int i=0; i<8; i++)
	  {
	       quad_matrix[i] = 0;
	  }
     }
    
/******************************************************************************
**
**     Method name:             getLat (class )
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           float
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public float getLat()
     {
	  return lat;
     }

/******************************************************************************
**
**     Method name:             getLon (class )
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           float  
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public float getLon()
     {
	  return lon;
     }

/******************************************************************************
**
**     Method name:             toString (class )
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String toString()
     {
	  return new String(Math.abs(lat) + ((lat < 0.0) ? " S" :" N") + 
			    ", " + Math.abs(lon)  + 
			    ((lon < 0.0) ? " W" :" E"));
     }

/******************************************************************************
**
**     Method name:             setPoint (class )
**
**     Purpose:                 
**    
**     Parameters:              Point p
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void setPoint(Point p)
     {
	 int width = size/2;
	 r = new Rectangle(p.x-width,p.y-width,size,size);
     }

/******************************************************************************
**
**     Method name:             getQuadMatrix (class )
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           int[]
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public int[] getQuadMatrix()
     {
	  return quad_matrix;
     }

/******************************************************************************
**
**     Method name:             quadMatrixToString (class )
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public String quadMatrixToString()
     {
	 String temp = toString();
	 for (int i=0; i<8; i++)
	 {
	      temp = temp + " " + i + " = " + quad_matrix[i];
	 }
	 return temp;
     }

/******************************************************************************
**
**     Method name:             getQuadrant (class )
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public int getQuadrant()
     {
	  for(int i=0; i<8; i+=2)
	  {
	       if (quad_matrix[labelorder[i]] == 0 && quad_matrix[labelorder[i+1]] == 0)
	       {
		    return (labelorder[i]*10)+labelorder[i+1];
	       }
	  }
	 
	  for(int i=0; i<8; i++)
	  {
	       if (quad_matrix[labelorder[i]] == 0)
	       {   
		    return labelorder[i];
	       }
	  }

	  int[] temp = new int[8];
	  int tmp;

	  for(int i=0; i<8; i++)
	  {
	       temp[i] = quad_matrix[labelorder[i]];
	  }

	  for(int i=0; i<8; i++)
	  {
	       for(int j=i+1; j<8; j++)
	       {
		    if (temp[i] > temp[j])
		    {
			 tmp = temp[j];
			 temp[j] = temp[i];
			 temp[i] = tmp;
		    }
	       }
	  }

	  return temp[0];
     }

/******************************************************************************
**
**     Method name:             isEmpty (class )
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           boolean
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public boolean isEmpty()
     {
	  for (int i=0; i<8; i++)
	  { 
	       if (quad_matrix[i] > 0)
	       {
		    return false;
	       }
	  }

	  return true;
     }

/******************************************************************************
**
**     Method name:            withinBoundary (class )
**
**     Purpose:                 
**    
**     Parameters:              Point pt
**
**     Return Values:           boolean 
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public boolean withinBoundary(Point pt)
     {
	 if ( r != null && r.contains(pt) )
	 {
	      return true;
	 }

	 return false;
     }

/******************************************************************************
**
**     Method name:             getDistance (class )
**
**     Purpose:                 
**    
**     Parameters:              Point pt
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public int getDistance(Point pt)
     {
	  int a = Math.abs(pt.x - (r.x + size/2));
	  int b = Math.abs(pt.y - (r.y + size/2));

	  int c = new Long(Math.round(
			   Math.sqrt(new Double(a*a +  
					  b*b).doubleValue()))).intValue();

          if ( c < size/2 )
	  {
	       return 0;
	  }
	  
	  return Math.abs(c-size/2);
     } 

/******************************************************************************
**
**     Method name:             drawSelf (class )
**
**     Purpose:                 
**    
**     Parameters:              Graphics g
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public void drawSelf(Graphics g)
     {
	  g.setColor(GeoPlot.fgcolor);
	  g.fillOval(r.x,r.y,size,size);
     }

/******************************************************************************
**
**     Method name:             getNodNum (class )
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public int getNodNum()
     {  
	  return nodNum;
     }

}
